@@ -1,5 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-# This module implements the base CCDData class.
+
+"""This module implements the base CCDData class."""
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
@@ -15,16 +17,16 @@ from astropy.nddata import (NDDataArray, StdDevUncertainty, NDUncertainty,
 from astropy.io import fits, registry
 from astropy import units as u
 from astropy import log
+from astropy.utils import minversion
 from astropy.wcs import WCS
 
+_ASTROPY_LT_1_2 = not minversion("astropy", "1.2")
+_ASTROPY_LT_1_3 = not minversion("astropy", "1.3")
+_ASTROPY_GT_2_0 = minversion("astropy", "2.0")
+
 # FIXME: Remove the content of the following "if" as soon as astropy 1.1 isn't
 # supported anymore. This is just a temporary workaround to fix the memory leak
 # described in https://github.com/astropy/astropy/issues/4825
-import astropy
-from distutils.version import LooseVersion
-
-_ASTROPY_LT_1_2 = LooseVersion(astropy.__version__) < LooseVersion('1.2')
-
 if _ASTROPY_LT_1_2:
 
     class ParentNDDataDescriptor(object):
@@ -38,10 +40,8 @@ if _ASTROPY_LT_1_2:
                     # to get the object the reference points to.
                     if isinstance(obj._parent_nddata, weakref.ref):
                         return obj._parent_nddata()
-                    else:
-                        log.info("parent_nddata should be a weakref to an "
-                                 "NDData object.")
-                        return obj._parent_nddata
+                    log.info("parent_nddata should be a weakref to an "
+                             "NDData object.")
                     return obj._parent_nddata
             except AttributeError:
                 raise MissingDataAssociationException(message)
@@ -169,11 +169,11 @@ class CCDData(NDDataArray):
 
     Methods
     -------
-    read(\*args, \**kwargs)
+    read(\\*args, \\**kwargs)
         ``Classmethod`` to create an CCDData instance based on a ``FITS`` file.
         This method uses :func:`fits_ccddata_reader` with the provided
         parameters.
-    write(\*args, \**kwargs)
+    write(\\*args, \\**kwargs)
         Writes the contents of the CCDData instance into a new ``FITS`` file.
         This method uses :func:`fits_ccddata_writer` with the provided
         parameters.
@@ -235,22 +235,7 @@ class CCDData(NDDataArray):
 
     @unit.setter
     def unit(self, value):
-        if value is not None:
-            self._unit = u.Unit(value)
-        else:
-            self._unit = None
-
-    @property
-    def shape(self):
-        return self.data.shape
-
-    @property
-    def size(self):
-        return self.data.size
-
-    @property
-    def dtype(self):
-        return self.data.dtype
+        self._unit = u.Unit(value)
 
     @property
     def header(self):
@@ -301,7 +286,7 @@ class CCDData(NDDataArray):
             self._uncertainty = value
 
     def to_hdu(self, hdu_mask='MASK', hdu_uncertainty='UNCERT',
-               hdu_flags=None):
+               hdu_flags=None, wcs_relax=True):
         """Creates an HDUList object from a CCDData object.
 
         Parameters
@@ -314,6 +299,13 @@ class CCDData(NDDataArray):
             Default is ``'MASK'`` for mask, ``'UNCERT'`` for uncertainty and
             ``None`` for flags.
 
+        wcs_relax : bool
+            Value of the ``relax`` parameter to use in converting the WCS to a
+            FITS header using `~astropy.wcs.WCS.to_header`. The common
+            ``CTYPE`` ``RA---TAN-SIP`` and ``DEC--TAN-SIP`` requires
+            ``relax=True`` for the ``-SIP`` part of the ``CTYPE`` to be
+            preserved.
+
         Raises
         -------
         ValueError
@@ -359,7 +351,7 @@ class CCDData(NDDataArray):
             # Note that until astropy/astropy#3967 is closed, the extend
             # will fail if there are comment cards in the WCS header but
             # not header.
-            wcs_header = self.wcs.to_header()
+            wcs_header = self.wcs.to_header(relax=wcs_relax)
             header.extend(wcs_header, useblanks=False, update=True)
         hdus = [fits.PrimaryHDU(self.data, header)]
 
@@ -615,6 +607,54 @@ class CCDData(NDDataArray):
             self.meta[key] = value
 
 
+# This needs to be importable by the tests...
+_KEEP_THESE_KEYWORDS_IN_HEADER = [
+    'JD-OBS',
+    'MJD-OBS',
+    'DATE-OBS'
+]
+
+
+def _generate_wcs_and_update_header(hdr):
+    """
+    Generate a WCS object from a header and remove the WCS-specific
+    keywords from the header.
+    Parameters
+    ----------
+    hdr : astropy.io.fits.header or other dict-like
+    Returns
+    -------
+    new_header, wcs
+    """
+
+    # Try constructing a WCS object.
+    try:
+        wcs = WCS(hdr)
+    except Exception as exc:
+        # Normally WCS only raises Warnings and doesn't fail but in rare
+        # cases (malformed header) it could fail...
+        log.info('An exception happened while extracting WCS informations from '
+                 'the Header.\n{}: {}'.format(type(exc).__name__, str(exc)))
+        return hdr, None
+    # Test for success by checking to see if the wcs ctype has a non-empty
+    # value, return None for wcs if ctype is empty.
+    if not wcs.wcs.ctype[0]:
+        return hdr, None
+
+    new_hdr = hdr.copy()
+    # If the keywords below are in the header they are also added to WCS.
+    # It seems like they should *not* be removed from the header, though.
+
+    wcs_header = wcs.to_header(relax=True)
+    for k in wcs_header:
+        if k not in _KEEP_THESE_KEYWORDS_IN_HEADER:
+            try:
+                new_hdr.remove(k)
+            except KeyError:
+                pass
+    return new_hdr, wcs
+
+
 def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
                         hdu_mask='MASK', hdu_flags=None, **kwd):
     """
@@ -694,7 +734,9 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
             for i in range(len(hdus)):
                 if hdus.fileinfo(i)['datSpan'] > 0:
                     hdu = i
-                    hdr = hdr + hdus[hdu].header
+                    comb_hdr = hdus[hdu].header.copy()
+                    comb_hdr.extend(hdr, unique=True)
+                    hdr = comb_hdr
                     log.info("first HDU with data is extension "
                              "{0}.".format(hdu))
                     break
@@ -714,12 +756,7 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, hdu_uncertainty='UNCERT',
                                                              fits_unit_string))
 
         use_unit = unit or fits_unit_string
-        # Try constructing a WCS object. This may generate a warning, but never
-        # an error.
-        wcs = WCS(hdr)
-        # Test for success by checking to see if the wcs ctype has a non-empty
-        # value.
-        wcs = wcs if wcs.wcs.ctype[0] else None
+        hdr, wcs = _generate_wcs_and_update_header(hdr)
         ccd_data = CCDData(hdus[hdu].data, meta=hdr, unit=use_unit,
                            mask=mask, uncertainty=uncertainty, wcs=wcs)
 
@@ -763,30 +800,39 @@ def fits_ccddata_writer(ccd_data, filename, hdu_mask='MASK',
                           hdu_flags=hdu_flags)
     hdu.writeto(filename, **kwd)
 
-
-# This should be be a tuple to ensure it isn't inadvertently changed elsewhere.
+# This should be be a tuple to ensure it isn't inadvertently changed
+# elsewhere.
 _recognized_fits_file_extensions = ('fit', 'fits', 'fts')
 
+if _ASTROPY_LT_1_3:
+    def is_fits(origin, filepath, fileobj, *args, **kwargs):
+        """
+        Wrapper around astropy.io.fits.connect.is_fits that handles the extra
+        extension.
 
-def is_fits(origin, filepath, fileobj, *args, **kwargs):
-    """
-    Wrapper around astropy.io.fits.connect.is_fits that handles the extra
-    extension.
+        Can be removed if fts is added to astropy.io as a recognized FITS
+        extension.
+        """
+        if ((filepath is not None) and
+                filepath.lower().endswith(('.fts', '.fts.gz'))):
 
-    Can be removed if fts is added to astropy.io as a recognized FITS
-    extension.
-    """
-    if ((filepath is not None) and
-            filepath.lower().endswith(('.fts', '.fts.gz'))):
+            return True
 
-        return True
+        else:
+            return fits.connect.is_fits(origin, filepath, fileobj,
+                                        *args, **kwargs)
+else:
+    is_fits = fits.connect.is_fits
 
-    else:
-        return fits.connect.is_fits(origin, filepath, fileobj, *args, **kwargs)
-
-registry.register_reader('fits', CCDData, fits_ccddata_reader)
-registry.register_writer('fits', CCDData, fits_ccddata_writer)
-registry.register_identifier('fits', CCDData, is_fits)
+if _ASTROPY_LT_1_3:
+    registry.register_reader('fits', CCDData, fits_ccddata_reader)
+    registry.register_writer('fits', CCDData, fits_ccddata_writer)
+    registry.register_identifier('fits', CCDData, is_fits)
+else:
+    with registry.delay_doc_updates(CCDData):
+        registry.register_reader('fits', CCDData, fits_ccddata_reader)
+        registry.register_writer('fits', CCDData, fits_ccddata_writer)
+        registry.register_identifier('fits', CCDData, is_fits)
 
 try:
     CCDData.read.__doc__ = fits_ccddata_reader.__doc__
@@ -797,3 +843,8 @@ try:
     CCDData.write.__doc__ = fits_ccddata_writer.__doc__
 except AttributeError:
     CCDData.write.__func__.__doc__ = fits_ccddata_writer.__doc__
+
+# CCDData moved to astropy core so we just import them from there (overwriting)
+# the classes defined here.
+if _ASTROPY_GT_2_0:
+    from astropy.nddata import fits_ccddata_reader, fits_ccddata_writer, CCDData
