@@ -1,5 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-# This module implements the combiner class.
+
+"""This module implements the combiner class."""
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
@@ -20,7 +22,7 @@ class Combiner(object):
     """
     A class for combining CCDData objects.
 
-    The Combiner class is used to combine together `~ccdproc.CCDData` objects
+    The Combiner class is used to combine together `~astropy.nddata.CCDData` objects
     including the method for combining the data, rejecting outlying data,
     and weighting used for combining frames.
 
@@ -37,13 +39,13 @@ class Combiner(object):
     Raises
     ------
     TypeError
-        If the ``ccd_list`` are not `~ccdproc.CCDData` objects, have different
+        If the ``ccd_list`` are not `~astropy.nddata.CCDData` objects, have different
         units, or are different shapes.
 
     Examples
     --------
     The following is an example of combining together different
-    `~ccdproc.CCDData` objects::
+    `~astropy.nddata.CCDData` objects::
 
         >>> import numpy as np
         >>> import astropy.units as u
@@ -53,7 +55,7 @@ class Combiner(object):
         >>> ccddata3 = CCDData(np.ones((4, 4)), unit=u.adu)
         >>> c = Combiner([ccddata1, ccddata2, ccddata3])
         >>> ccdall = c.average_combine()
-        >>> ccdall
+        >>> ccdall  # doctest: +FLOAT_CMP
         CCDData([[ 0.66666667,  0.66666667,  0.66666667,  0.66666667],
                  [ 0.66666667,  0.66666667,  0.66666667,  0.66666667],
                  [ 0.66666667,  0.66666667,  0.66666667,  0.66666667],
@@ -116,7 +118,7 @@ class Combiner(object):
     @property
     def weights(self):
         """
-        Weights used when combining the `~ccdproc.CCDData` objects.
+        Weights used when combining the `~astropy.nddata.CCDData` objects.
 
         Parameters
         ----------
@@ -311,7 +313,7 @@ class Combiner(object):
         """
         Median combine a set of arrays.
 
-        A `~ccdproc.CCDData` object is returned with the data property set to
+        A `~astropy.nddata.CCDData` object is returned with the data property set to
         the median of the arrays. If the data was masked or any data have been
         rejected, those pixels will not be included in the median. A mask will
         be returned, and if a pixel has been rejected in all images, it will be
@@ -335,7 +337,7 @@ class Combiner(object):
 
         Returns
         -------
-        combined_image: `~ccdproc.CCDData`
+        combined_image: `~astropy.nddata.CCDData`
             CCDData object based on the combined input of CCDData objects.
 
         Warnings
@@ -387,7 +389,7 @@ class Combiner(object):
         """
         Average combine together a set of arrays.
 
-        A `~ccdproc.CCDData` object is returned with the data property
+        A `~astropy.nddata.CCDData` object is returned with the data property
         set to the average of the arrays. If the data was masked or any
         data have been rejected, those pixels will not be included in the
         average. A mask will be returned, and if a pixel has been
@@ -410,7 +412,7 @@ class Combiner(object):
 
         Returns
         -------
-        combined_image: `~ccdproc.CCDData`
+        combined_image: `~astropy.nddata.CCDData`
             CCDData object based on the combined input of CCDData objects.
         """
         if scale_to is not None:
@@ -447,6 +449,74 @@ class Combiner(object):
         # return the combined image
         return combined_image
 
+    def sum_combine(self, sum_func=ma.sum, scale_to=None,
+                    uncertainty_func=ma.std):
+        """
+        Sum combine together a set of arrays.
+
+        A `~astropy.nddata.CCDData` object is returned with the data property
+        set to the sum of the arrays. If the data was masked or any
+        data have been rejected, those pixels will not be included in the
+        sum. A mask will be returned, and if a pixel has been
+        rejected in all images, it will be masked. The uncertainty of
+        the combined image is set by the multiplication of summation of
+        standard deviation of the input by square root of number of images.
+        Because sum_combine returns 'pure sum' with masked pixels ignored, if
+        re-scaled sum is needed, average_combine have to be used with
+        multiplication by number of images combined.
+
+        Parameters
+        ----------
+        sum_func : function, optional
+            Function to calculate the sum. Defaults to
+            `numpy.ma.sum`.
+
+        scale_to : float or None, optional
+            Scaling factor used in the sum combined image. If given,
+            it overrides `scaling`. Defaults to ``None``.
+
+        uncertainty_func : function, optional
+            Function to calculate uncertainty. Defaults to `numpy.ma.std`.
+
+        Returns
+        -------
+        combined_image: `~astropy.nddata.CCDData`
+            CCDData object based on the combined input of CCDData objects.
+        """
+        if scale_to is not None:
+            scalings = scale_to
+        elif self.scaling is not None:
+            scalings = self.scaling
+        else:
+            scalings = 1.0
+
+        # set up the data
+        data = sum_func(scalings * self.data_arr, axis=0)
+
+        # set up the mask
+        masked_values = self.data_arr.mask.sum(axis=0)
+        mask = (masked_values == len(self.data_arr))
+
+        # set up the deviation
+        uncertainty = uncertainty_func(self.data_arr, axis=0)
+        # Divide uncertainty by the number of pixel (#309)
+        uncertainty /= np.sqrt(len(self.data_arr) - masked_values)
+        # Convert uncertainty to plain numpy array (#351)
+        uncertainty = np.asarray(uncertainty)
+        # Multiply uncertainty by square root of the number of images
+        uncertainty *= len(self.data_arr) - masked_values
+
+        # create the combined image with a dtype that matches the combiner
+        combined_image = CCDData(np.asarray(data.data, dtype=self.dtype),
+                                 mask=mask, unit=self.unit,
+                                 uncertainty=StdDevUncertainty(uncertainty))
+
+        # update the meta data
+        combined_image.meta['NCOMBINE'] = len(self.data_arr)
+
+        # return the combined image
+        return combined_image
+
 
 def combine(img_list, output_file=None,
             method='average', weights=None, scale=None, mem_limit=16e9,
@@ -461,8 +531,8 @@ def combine(img_list, output_file=None,
 
     Parameters
     -----------
-    img_list : list or str
-        A list of fits filenames or `~ccdproc.CCDData` objects that will be
+    img_list : `numpy.ndarray`, list or str
+        A list of fits filenames or `~astropy.nddata.CCDData` objects that will be
         combined together. Or a string of fits filenames separated by comma
         ",".
 
@@ -476,6 +546,7 @@ def combine(img_list, output_file=None,
 
         - ``'average'`` : To combine by calculating the average.
         - ``'median'`` : To combine by calculating the median.
+        - ``'sum'`` : To combine by calculating the sum.
 
         Default is ``'average'``.
 
@@ -547,21 +618,23 @@ def combine(img_list, output_file=None,
         Default is ``None``.
 
     combine_uncertainty_function : callable, None, optional
-        If ``None`` use the default uncertainty func when using average or
-        median combine, otherwise use the function provided.
+        If ``None`` use the default uncertainty func when using average, median or
+        sum combine, otherwise use the function provided.
         Default is ``None``.
 
-    ccdkwargs : Other keyword arguments for `ccdproc.fits_ccddata_reader`.
+    ccdkwargs : Other keyword arguments for `astropy.nddata.fits_ccddata_reader`.
 
     Returns
     -------
-    combined_image : `~ccdproc.CCDData`
+    combined_image : `~astropy.nddata.CCDData`
         CCDData object based on the combined input of CCDData objects.
     """
     if not isinstance(img_list, list):
-        # If not a list, check whether it is a string of filenames separated
-        # by comma
-        if isinstance(img_list, str) and (',' in img_list):
+        # If not a list, check whether it is a numpy ndarray or string of
+        # filenames separated by comma
+        if isinstance(img_list, np.ndarray):
+            img_list = img_list.tolist()
+        elif isinstance(img_list, str) and (',' in img_list):
             img_list = img_list.split(',')
         else:
             raise ValueError(
@@ -572,6 +645,8 @@ def combine(img_list, output_file=None,
         combine_function = 'average_combine'
     elif method == 'median':
         combine_function = 'median_combine'
+    elif method == 'sum':
+        combine_function = 'sum_combine'
     else:
         raise ValueError("unrecognised combine method : {0}.".format(method))
 
