@@ -1,5 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-# This module implements the base CCDPROC functions
+
+"""This module implements the base CCDPROC functions"""
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
@@ -23,13 +25,14 @@ from scipy import ndimage
 from .ccddata import CCDData
 from .utils.slices import slice_from_string
 from .log_meta import log_to_metadata
+from .extern.bitfield import bitfield_to_boolean_mask as _bitfield_to_boolean_mask
 
 __all__ = ['background_deviation_box', 'background_deviation_filter',
            'ccd_process', 'cosmicray_median', 'cosmicray_lacosmic',
            'create_deviation', 'flat_correct', 'gain_correct', 'rebin',
            'sigma_func', 'subtract_bias', 'subtract_dark', 'subtract_overscan',
            'transform_image', 'trim_image', 'wcs_project', 'Keyword',
-           'median_filter', 'ccdmask']
+           'median_filter', 'ccdmask', 'bitfield_to_boolean_mask']
 
 # The dictionary below is used to translate actual function names to names
 # that are FITS compliant, i.e. 8 characters or less.
@@ -56,7 +59,7 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
                 gain=None, readnoise=None, oscan_median=True, oscan_model=None,
                 min_value=None, dark_exposure=None, data_exposure=None,
                 exposure_key=None, exposure_unit=None,
-                dark_scale=False):
+                dark_scale=False, gain_corrected=True):
     """Perform basic processing on ccd data.
 
     The following steps can be included:
@@ -70,14 +73,14 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
     * subtraction of a dark frame (:func:`subtract_dark`)
     * correction of flat field (:func:`flat_correct`)
 
-    The task returns a processed `~ccdproc.CCDData` object.
+    The task returns a processed `~astropy.nddata.CCDData` object.
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         Frame to be reduced.
 
-    oscan : `~ccdproc.ccddata.CCDData`, str or None, optional
+    oscan : `~astropy.nddata.CCDData`, str or None, optional
         For no overscan correction, set to None. Otherwise provide a region
         of ccd from which the overscan is extracted, using the FITS
         conventions for index order and index start, or a
@@ -94,22 +97,22 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
         If True, create an uncertainty array for ccd.
         Default is ``False``.
 
-    master_bias : `~ccdproc.CCDData` or None, optional
+    master_bias : `~astropy.nddata.CCDData` or None, optional
         A master bias frame to be subtracted from ccd. The unit of the
         master bias frame should match the unit of the image **after
-        gain correction**.
+        gain correction** if ``gain_corrected`` is True.
         Default is ``None``.
 
-    dark_frame : `~ccdproc.CCDData` or None, optional
+    dark_frame : `~astropy.nddata.CCDData` or None, optional
         A dark frame to be subtracted from the ccd. The unit of the
         master dark frame should match the unit of the image **after
-        gain correction**.
+        gain correction** if ``gain_corrected`` is True.
         Default is ``None``.
 
-    master_flat : `~ccdproc.CCDData` or None, optional
+    master_flat : `~astropy.nddata.CCDData` or None, optional
         A master flat frame to be divided into ccd. The unit of the
         master flat frame should match the unit of the image **after
-        gain correction**.
+        gain correction** if ``gain_corrected`` is True.
         Default is ``None``.
 
     bad_pixel_mask : `numpy.ndarray` or None, optional
@@ -164,9 +167,13 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
         If True, scale the dark frame by the exposure times.
         Default is ``False``.
 
+    gain_corrected : bool, optional
+        If True, the ``master_bias``, ``master_flat``, and ``dark_frame``
+        have already been gain corrected.  Default is ``True``.
+
     Returns
     -------
-    occd : `~ccdproc.CCDData`
+    occd : `~astropy.nddata.CCDData`
         Reduded ccd.
 
     Examples
@@ -223,13 +230,12 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
         raise TypeError('bad_pixel_mask is not None or numpy.ndarray.')
 
     # apply the gain correction
-    if isinstance(gain, Quantity):
-        nccd = gain_correct(nccd, gain)
-    elif gain is None:
-        pass
-    else:
+    if not (gain is None or isinstance(gain, Quantity)):
         raise TypeError('gain is not None or astropy.units.Quantity.')
 
+    if gain is not None and gain_corrected:
+        nccd = gain_correct(nccd, gain)
+
     # subtracting the master bias
     if isinstance(master_bias, CCDData):
         nccd = subtract_bias(nccd, master_bias)
@@ -261,6 +267,10 @@ def ccd_process(ccd, oscan=None, trim=None, error=False, master_bias=None,
         raise TypeError(
             'master_flat is not None or a CCDData object.')
 
+    # apply the gain correction only at the end if gain_corrected is False
+    if gain is not None and not gain_corrected:
+        nccd = gain_correct(nccd, gain)
+
     return nccd
 
 
@@ -274,7 +284,7 @@ def create_deviation(ccd_data, gain=None, readnoise=None):
 
     Parameters
     ----------
-    ccd_data : `~ccdproc.CCDData`
+    ccd_data : `~astropy.nddata.CCDData`
         Data whose deviation will be calculated.
 
     gain : `~astropy.units.Quantity` or None, optional
@@ -298,7 +308,7 @@ def create_deviation(ccd_data, gain=None, readnoise=None):
 
     Returns
     -------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         CCDData object with uncertainty created; uncertainty is in the same
         units as the data in the parameter ``ccd_data``.
 
@@ -339,10 +349,10 @@ def subtract_overscan(ccd, overscan=None, overscan_axis=1, fits_section=None,
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         Data to have overscan frame corrected.
 
-    overscan : `~ccdproc.CCDData` or None, optional
+    overscan : `~astropy.nddata.CCDData` or None, optional
         Slice from ``ccd`` that contains the overscan. Must provide either
         this argument or ``fits_section``, but not both.
         Default is ``None``.
@@ -382,7 +392,7 @@ def subtract_overscan(ccd, overscan=None, overscan_axis=1, fits_section=None,
 
     Returns
     -------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         CCDData object with overscan subtracted.
 
     Notes
@@ -476,7 +486,7 @@ def trim_image(ccd, fits_section=None):
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         CCD image to be trimmed, sliced if desired.
 
     fits_section : str or None, optional
@@ -488,7 +498,7 @@ def trim_image(ccd, fits_section=None):
 
     Returns
     -------
-    trimmed_ccd : `~ccdproc.CCDData`
+    trimmed_ccd : `~astropy.nddata.CCDData`
         Trimmed image.
 
     Examples
@@ -539,17 +549,17 @@ def subtract_bias(ccd, master):
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         Image from which bias will be subtracted.
 
-    master : `~ccdproc.CCDData`
+    master : `~astropy.nddata.CCDData`
         Master image to be subtracted from ``ccd``.
 
     {log}
 
     Returns
     -------
-    result : `~ccdproc.CCDData`
+    result : `~astropy.nddata.CCDData`
         CCDData object with bias subtracted.
     """
 
@@ -576,10 +586,10 @@ def subtract_dark(ccd, master, dark_exposure=None, data_exposure=None,
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         Image from which dark will be subtracted.
 
-    master : `~ccdproc.CCDData`
+    master : `~astropy.nddata.CCDData`
         Dark image.
 
     dark_exposure : `~astropy.units.Quantity` or None, optional
@@ -609,7 +619,7 @@ def subtract_dark(ccd, master, dark_exposure=None, data_exposure=None,
 
     Returns
     -------
-    result : `~ccdproc.CCDData`
+    result : `~astropy.nddata.CCDData`
         Dark-subtracted image.
     """
     if not (isinstance(ccd, CCDData) and isinstance(master, CCDData)):
@@ -678,7 +688,7 @@ def gain_correct(ccd, gain, gain_unit=None):
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
       Data to have gain corrected.
 
     gain : `~astropy.units.Quantity` or `~ccdproc.Keyword`
@@ -693,7 +703,7 @@ def gain_correct(ccd, gain, gain_unit=None):
 
     Returns
     -------
-    result : `~ccdproc.CCDData`
+    result : `~astropy.nddata.CCDData`
       CCDData object with gain corrected.
     """
     if isinstance(gain, Keyword):
@@ -709,17 +719,18 @@ def gain_correct(ccd, gain, gain_unit=None):
 
 
 @log_to_metadata
-def flat_correct(ccd, flat, min_value=None):
+def flat_correct(ccd, flat, min_value=None, norm_value=None):
     """Correct the image for flat fielding.
 
-    The flat field image is normalized by its mean before flat correcting.
+    The flat field image is normalized by its mean or a user-supplied value
+    before flat correcting.
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         Data to be transformed.
 
-    flat : `~ccdproc.CCDData`
+    flat : `~astropy.nddata.CCDData`
         Flatfield to apply to the data.
 
     min_value : float or None, optional
@@ -728,11 +739,17 @@ def flat_correct(ccd, flat, min_value=None):
         will replace all values in the flat by the min_value.
         Default is ``None``.
 
+    norm_value : float or None, optional
+        If not ``None``, normalize flat field by this argument rather than the
+        mean of the image. This allows fixing several different flat fields to
+        have the same scale. If this value is negative or 0, a ``ValueError``
+        is raised. Default is ``None``.
+
     {log}
 
     Returns
     -------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         CCDData object with flat corrected.
     """
     # Use the min_value to replace any values in the flat
@@ -742,8 +759,18 @@ def flat_correct(ccd, flat, min_value=None):
         flat_min.data[flat_min.data < min_value] = min_value
         use_flat = flat_min
 
+    # If a norm_value was input and is positive, use it to scale the flat
+    if norm_value is not None and norm_value > 0:
+        flat_mean_val = norm_value
+    elif norm_value is not None:
+        # norm_value was set to a bad value
+        raise ValueError('norm_value must be greater than zero.')
+    else:
+        # norm_value was not set, use mean of the image.
+        flat_mean_val = use_flat.data.mean()
+
     # Normalize the flat.
-    flat_mean = use_flat.data.mean() * use_flat.unit
+    flat_mean = flat_mean_val * use_flat.unit
     flat_normed = use_flat.divide(flat_mean)
 
     # divide through the flat
@@ -762,8 +789,8 @@ def transform_image(ccd, transform_func, **kwargs):
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
-        Data to be flatfield corrected.
+    ccd : `~astropy.nddata.CCDData`
+        Data to be transformed.
 
     transform_func : callable
         Function to be used to transform the data, mask and uncertainty.
@@ -775,7 +802,7 @@ def transform_image(ccd, transform_func, **kwargs):
 
     Returns
     -------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         A transformed CCDData object.
 
     Notes
@@ -841,7 +868,7 @@ def wcs_project(ccd, target_wcs, target_shape=None, order='bilinear'):
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         Data to be projected.
 
     target_wcs : `~astropy.wcs.WCS` object
@@ -866,7 +893,7 @@ def wcs_project(ccd, target_wcs, target_shape=None, order='bilinear'):
 
     Returns
     -------
-    ccd : `~ccdproc.CCDData`
+    ccd : `~astropy.nddata.CCDData`
         A transformed CCDData object.
     """
     from reproject import reproject_interp
@@ -922,7 +949,7 @@ def sigma_func(arr, axis=None):
 
     Parameters
     ----------
-    arr : `~ccdproc.CCDData` or `numpy.ndarray`
+    arr : `~astropy.nddata.CCDData` or `numpy.ndarray`
         Array whose deviation is to be calculated.
 
     axis : int, tuple of ints or None, optional
@@ -1064,7 +1091,7 @@ def rebin(ccd, newshape):
 
     Parameters
     ----------
-    data : `~ccdproc.CCDData` or `numpy.ndarray`
+    ccd : `~astropy.nddata.CCDData` or `numpy.ndarray`
         Data to rebin.
 
     newshape : tuple
@@ -1072,7 +1099,7 @@ def rebin(ccd, newshape):
 
     Returns
     -------
-    output : `~ccdproc.CCDData` or `numpy.ndarray`
+    output : `~astropy.nddata.CCDData` or `numpy.ndarray`
         An array with the new shape. It will have the same type as the input
         object.
 
@@ -1080,7 +1107,7 @@ def rebin(ccd, newshape):
     ------
     TypeError
         A type error is raised if data is not an `numpy.ndarray` or
-        `~ccdproc.CCDData`.
+        `~astropy.nddata.CCDData`.
 
     ValueError
         A value error is raised if the dimension of the new shape is not equal
@@ -1098,14 +1125,14 @@ def rebin(ccd, newshape):
     --------
     Given an array that is 100x100::
 
-        >>> import numpy as np
-        >>> from astropy import units as u
-        >>> arr1 = CCDData(np.ones([10, 10]), unit=u.adu)
+        import numpy as np
+        from astropy import units as u
+        arr1 = CCDData(np.ones([10, 10]), unit=u.adu)
 
     The syntax for rebinning an array to a shape
     of (20,20) is::
 
-        >>> rebinned = rebin(arr1, (20,20))
+        rebin(arr1, (20,20))
     """
     # check to see that is in a nddata type
     if isinstance(ccd, np.ndarray):
@@ -1236,8 +1263,8 @@ def _blkavg(data, newshape):
 def median_filter(data, *args, **kwargs):
     """See `scipy.ndimage.median_filter` for arguments.
 
-    If the ``data`` is a `~ccdproc.CCDData` object the result will be another
-    `~ccdproc.CCDData` object with the median filtered data as ``data`` and
+    If the ``data`` is a `~astropy.nddata.CCDData` object the result will be another
+    `~astropy.nddata.CCDData` object with the median filtered data as ``data`` and
     copied ``unit`` and ``meta``.
     """
     if isinstance(data, CCDData):
@@ -1264,7 +1291,7 @@ def cosmicray_lacosmic(ccd, sigclip=4.5, sigfrac=0.3,
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData` or `numpy.ndarray`
+    ccd : `~astropy.nddata.CCDData` or `numpy.ndarray`
         Data to have cosmic ray cleaned.
 
     sigclip : float, optional
@@ -1365,9 +1392,9 @@ def cosmicray_lacosmic(ccd, sigclip=4.5, sigfrac=0.3,
 
     Returns
     -------
-    nccd : `~ccdproc.CCDData` or `numpy.ndarray`
+    nccd : `~astropy.nddata.CCDData` or `numpy.ndarray`
         An object of the same type as ccd is returned. If it is a
-        `~ccdproc.CCDData`, the mask attribute will also be updated with
+        `~astropy.nddata.CCDData`, the mask attribute will also be updated with
         areas identified with cosmic rays masked.
 
     crmask : `numpy.ndarray`
@@ -1396,7 +1423,7 @@ def cosmicray_lacosmic(ccd, sigclip=4.5, sigfrac=0.3,
        with the bad pixels replaced by the local median from a box of 11
        pixels; and it would return a mask indicating the bad pixels.
 
-    2) Given an `~ccdproc.CCDData` object with an uncertainty frame, the syntax
+    2) Given an `~astropy.nddata.CCDData` object with an uncertainty frame, the syntax
        for running cosmicrar_lacosmic would be:
 
        >>> newccd = cosmicray_lacosmic(ccd, sigclip=5)   # doctest: +SKIP
@@ -1453,7 +1480,7 @@ def cosmicray_median(ccd, error_image=None, thresh=5, mbox=11, gbox=0,
 
     Parameters
     ----------
-    ccd : `~ccdproc.CCDData`, `numpy.ndarray` or `numpy.ma.MaskedArray`
+    ccd : `~astropy.nddata.CCDData`, `numpy.ndarray` or `numpy.ma.MaskedArray`
         Data to have cosmic ray cleaned.
 
     thresh : float, optional
@@ -1485,9 +1512,9 @@ def cosmicray_median(ccd, error_image=None, thresh=5, mbox=11, gbox=0,
 
     Returns
     -------
-    nccd : `~ccdproc.CCDData` or `numpy.ndarray`
+    nccd : `~astropy.nddata.CCDData` or `numpy.ndarray`
         An object of the same type as ccd is returned. If it is a
-        `~ccdproc.CCDData`, the mask attribute will also be updated with
+        `~astropy.nddata.CCDData`, the mask attribute will also be updated with
         areas identified with cosmic rays masked.
 
     nccd : `numpy.ndarray`
@@ -1508,7 +1535,7 @@ def cosmicray_median(ccd, error_image=None, thresh=5, mbox=11, gbox=0,
        with the bad pixels replaced by the local median from a box of 11
        pixels; and it would return a mask indicating the bad pixels.
 
-    2) Given an `~ccdproc.CCDData` object with an uncertainty frame, the syntax
+    2) Given an `~astropy.nddata.CCDData` object with an uncertainty frame, the syntax
        for running cosmicray_median would be:
 
        >>> newccd = cosmicray_median(ccd, thresh=5, mbox=11,
@@ -1590,15 +1617,16 @@ def ccdmask(ratio, findbadcolumns=False, byblocks=False, ncmed=7, nlmed=7,
 
     Parameters
     ----------
-    ratio : `~ccdproc.CCDData`
+    ratio : `~astropy.nddata.CCDData`
         Data to used to form mask.  Typically this is the ratio of two flat
         field images.
 
-    findbadcolumns: `bool`, optional
+    findbadcolumns : `bool`, optional
         If set to True, the code will search for bad column sections.  Note
         that this treats columns as special and breaks symmetry between lines
         and columns and so is likely only appropriate for detectors which have
         readout directions.
+        Default is ``False``.
 
     byblocks : `bool`, optional
         If set to true, the code will divide the image up in to blocks of size
@@ -1608,25 +1636,30 @@ def ccdmask(ratio, findbadcolumns=False, byblocks=False, ncmed=7, nlmed=7,
         to generate a running box version of the standard
         deviation estimate and use that value for the standard deviation at
         each pixel.
+        Default is ``False``.
 
-    ncmed, nlmed: `int`, optional
+    ncmed, nlmed : `int`, optional
         The column and line size of the moving median rectangle used to
         estimate the uncontaminated local signal. The column median size should
         be at least 3 pixels to span single bad columns.
+        Default is ``7``.
 
-    ncsig, nlsig: `int`, optional
+    ncsig, nlsig : `int`, optional
         The column and line size of regions used to estimate the uncontaminated
         local sigma using a percentile. The size of the box should contain of
         order 100 pixels or more.
+        Default is ``15``.
 
-    lsigma, hsigma: `float`, optional
+    lsigma, hsigma : `float`, optional
         Positive sigma factors to use for selecting pixels below and above the
         median level based on the local percentile sigma.
+        Default is ``9``.
 
-    ngood: `int`, optional
+    ngood : `int`, optional
         Gaps of undetected pixels along the column direction of length less
         than this amount are also flagged as bad pixels, if they are between
         pixels masked in that column.
+        Default is ``5``.
 
     Returns
     -------
@@ -1740,6 +1773,95 @@ def ccdmask(ratio, findbadcolumns=False, byblocks=False, ncmed=7, nlmed=7,
     return mask
 
 
+def bitfield_to_boolean_mask(bitfield, ignore_bits=0, flip_bits=None):
+    """Convert an integer bit field to a boolean mask.
+
+    Parameters
+    ----------
+    bitfield : `numpy.ndarray` of integer dtype
+        The array of bit flags.
+
+    ignore_bits : int, None or str, optional
+        The bits to ignore when converting the bitfield.
+
+        - If it's an integer it's binary representation is interpreted as the
+          bits to ignore. ``0`` means that all bit flags are taken into
+          account while a binary representation of all ``1`` means that all
+          flags would be ignored.
+        - If it's ``None`` then all flags are ignored
+        - If it's a string then it must be a ``,`` or ``+`` separated string
+          of integers that bits to ignore. If the string starts with an ``~``
+          the integers are interpreted as **the only flags** to take into
+          account.
+
+        Default is ``0``.
+
+    Returns
+    -------
+    mask : `numpy.ndarray` of boolean dtype
+        The bitfield converted to a boolean mask that can be used for
+        `numpy.ma.MaskedArray` or `~astropy.nddata.CCDData`.
+
+    Examples
+    --------
+    Bitfields (or data quality arrays) are integer arrays where the binary
+    representation of the values indicates whether a specific flag is set or
+    not. The convention is that a value of ``0`` represents a **good value**
+    and a value that is ``!= 0`` represents a value that is in some (or
+    multiple) ways considered a **bad value**. The ``bitfield_to_boolean_mask``
+    function can be used by default to create a boolean mask wherever any bit
+    flag is set::
+
+        >>> import ccdproc
+        >>> import numpy as np
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8))
+        array([False,  True,  True,  True,  True,  True,  True,  True], dtype=bool)
+
+    To ignore all bit flags ``ignore_bits=None`` can be used::
+
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=None)
+        array([False, False, False, False, False, False, False, False], dtype=bool)
+
+    To ignore only specific bit flags one can use a ``list`` of bits flags to
+    ignore::
+
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=[1, 4])
+        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+
+    There are some equivalent ways::
+
+        >>> # pass in the sum of the "ignore_bits" directly
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=5)  # 1 + 4
+        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        >>> # use a comma seperated string of integers
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='1, 4')
+        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        >>> # use a + seperated string of integers
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='1+4')
+        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+
+    Instead of directly specifying the **bits flags to ignore** one can also
+    pass in the **only bits that shouldn't be ignored** by prepending a ``~``
+    to the string of ``ignore_bits`` (or if it's not a string in
+    ``ignore_bits`` one can set ``flip_bits=True``)::
+
+        >>> # ignore all bit flags except the one for 2.
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='~(2)')
+        array([False, False,  True,  True, False, False,  True,  True], dtype=bool)
+        >>> # ignore all bit flags except the one for 1, 8 and 32.
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits='~(1, 8, 32)')
+        array([False,  True, False,  True, False,  True, False,  True], dtype=bool)
+
+        >>> # Equivalent for a list using flip_bits.
+        >>> ccdproc.bitfield_to_boolean_mask(np.arange(8), ignore_bits=[1, 8, 32], flip_bits=True)
+        array([False,  True, False,  True, False,  True, False,  True], dtype=bool)
+
+    """
+    return _bitfield_to_boolean_mask(
+        bitfield, ignore_bits, flip_bits=flip_bits,
+        good_mask_value=False, dtype=bool)
+
+
 class Keyword(object):
     """
     """
