@@ -1,6 +1,9 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
+
 from __future__ import (print_function, division, absolute_import,
                         unicode_literals)
 
+from collections import OrderedDict
 import fnmatch
 from os import listdir, path
 import logging
@@ -8,20 +11,23 @@ import logging
 import numpy as np
 import numpy.ma as ma
 
-from astropy.table import Table
+from astropy.table import Table, MaskedColumn
 import astropy.io.fits as fits
 from astropy.extern import six
+from astropy.utils import minversion
 
 import warnings
 from astropy.utils.exceptions import AstropyUserWarning
 
-from .ccddata import fits_ccddata_reader
+from .ccddata import fits_ccddata_reader, _recognized_fits_file_extensions
 
 logger = logging.getLogger(__name__)
 
 __all__ = ['ImageFileCollection']
 __doctest_skip__ = ['*']
 
+_ASTROPY_LT_1_3 = not minversion("astropy", "1.3")
+
 
 class ImageFileCollection(object):
     """
@@ -53,11 +59,29 @@ class ImageFileCollection(object):
         list.
         Default is ``None``.
 
+        .. deprecated:: 1.3
+
     filenames: str, list of str, or None, optional
         List of the names of FITS files which will be added to the collection.
         The filenames are assumed to be in ``location``.
         Default is ``None``.
 
+    glob_include: str or None, optional
+        Unix-style filename pattern to select filenames to include in the file
+        collection. Can be used in conjunction with ``glob_exclude`` to
+        easily select subsets of files in the target directory.
+        Default is ``None``.
+
+    glob_exclude: str or None, optional
+        Unix-style filename pattern to select filenames to exclude from the
+        file collection. Can be used in conjunction with ``glob_include`` to
+        easily select subsets of files in the target directory.
+        Default is ``None``.
+
+     ext: str or int, optional
+         The extension from which the header and data will be read in all files.
+         Default is ``0``.
+
     Raises
     ------
     ValueError
@@ -65,17 +89,33 @@ class ImageFileCollection(object):
         value.
     """
     def __init__(self, location=None, keywords=None, info_file=None,
-                 filenames=None):
+                 filenames=None, glob_include=None, glob_exclude=None, ext=0):
+
+        if info_file is not None:
+            warnings.warn("The 'info_file' argument is deprecated and will be "
+                          "removed in a future version", DeprecationWarning)
+
+        # Include or exclude files from the collection based on glob pattern
+        # matching - has to go above call to _get_files()
+        if glob_exclude is not None:
+            glob_exclude = str(glob_exclude)  # some minimal validation
+        self._glob_exclude = glob_exclude
+
+        if glob_include is not None:
+            glob_include = str(glob_include)
+        self._glob_include = glob_include
+
         self._location = location
         self._filenames = filenames
         self._files = []
+        self._info_file = info_file
         if location:
             self._files = self._get_files()
 
         if self._files == []:
             warnings.warn("no FITS files in the collection.",
                           AstropyUserWarning)
-        self._summary_info = {}
+        self._summary = {}
         if keywords is None:
             if info_file is not None:
                 # Default to empty list so that keywords will be populated
@@ -90,11 +130,9 @@ class ImageFileCollection(object):
             except (AttributeError, TypeError):
                 info_path = info_file
             try:
-                self._summary_info = Table.read(info_path,
-                                                format='ascii',
-                                                delimiter=',')
-                self._summary_info = Table(self._summary_info,
-                                           masked=True)
+                self._summary = Table.read(info_path, format='ascii',
+                                           delimiter=',')
+                self._summary = Table(self._summary, masked=True)
             except IOError:
                 if location:
                     logger.warning('unable to open table file %s, will try '
@@ -109,9 +147,55 @@ class ImageFileCollection(object):
         # is always *some* value.
         self._all_keywords = False
 
+        self._ext = ext
+
         if keywords:
             self.keywords = keywords
 
+    def __repr__(self):
+        if self.location is None:
+            location = ""
+        else:
+            location = "location={!r}".format(self.location)
+
+        if self._all_keywords:
+            kw = ""
+        else:
+            kw = "keywords={!r}".format(self.keywords[1:])
+
+        if self._info_file is None:
+            infofile = ''
+        else:
+            infofile = "info_file={!r}".format(self._info_file)
+
+        if self.glob_exclude is None:
+            glob_exclude = ''
+        else:
+            glob_exclude = "glob_exclude={!r}".format(self.glob_exclude)
+
+        if self.glob_include is None:
+            glob_include = ''
+        else:
+            glob_include = "glob_include={!r}".format(self.glob_include)
+
+        if self.ext == 0:
+            ext = ""
+        else:
+            ext = "ext={}".format(self.ext)
+
+        if self._filenames is None:
+            filenames = ""
+        else:
+            filenames = "filenames={}".format(self._filenames)
+
+        params = [location, kw, infofile, filenames, glob_include, glob_exclude, ext]
+        params = ', '.join([p for p in params if p])
+
+        str_repr = "{self.__class__.__name__}({params})".format(
+            self=self, params=params)
+
+        return str_repr
+
     @property
     def summary(self):
         """
@@ -135,19 +219,24 @@ class ImageFileCollection(object):
         collection then the order of the columns is the order of the
         keywords.
         """
-        return self._summary_info
+        return self._summary
 
     @property
     def summary_info(self):
         """
-        Deprecated -- use `summary` instead -- `~astropy.table.Table` of values
-        of FITS keywords for files in the collection.
+        `~astropy.table.Table` of values of FITS keywords for files in the
+        collection.
 
         Each keyword is a column heading. In addition, there is a column
         called 'file' that contains the name of the FITS file. The directory
         is not included as part of that name.
+
+        .. deprecated:: 0.4
         """
-        return self._summary_info
+        warnings.warn('"summary_info" is deprecated and will be removed in '
+                      'a future version. Use the "summary" attribute instead.',
+                      AstropyUserWarning)
+        return self._summary
 
     @property
     def location(self):
@@ -163,18 +252,21 @@ class ImageFileCollection(object):
 
         Setting the keywords causes the summary table to be regenerated unless
         the new keywords are a subset of the old.
+
+        .. versionchanged:: 1.3
+            Added ``deleter`` for ``keywords`` property.
         """
-        if self.summary_info:
-            return self.summary_info.keys()
+        if self.summary:
+            return self.summary.keys()
         else:
             return []
 
     @keywords.setter
-    def keywords(self, keywords=None):
-        # since keywords are drawn from self.summary_info, setting
-        # summary_info sets the keywords.
+    def keywords(self, keywords):
+        # since keywords are drawn from self.summary, setting
+        # summary sets the keywords.
         if keywords is None:
-            self._summary_info = []
+            self._summary = []
             return
 
         if keywords == '*':
@@ -184,30 +276,36 @@ class ImageFileCollection(object):
 
         logging.debug('keywords in setter before pruning: %s.', keywords)
 
-        # remove duplicates and force a copy
-        new_keys = list(set(keywords))
+        # remove duplicates and force a copy so we can sort the items later
+        # by their given position.
+        new_keys_set = set(keywords)
+        new_keys_lst = list(new_keys_set)
+        new_keys_set.add('file')
 
-        logging.debug('keywords after pruning %s.', new_keys)
+        logging.debug('keywords after pruning %s.', new_keys_lst)
 
-        full_new_keys = list(set(new_keys))
-        full_new_keys.append('file')
-        full_new_set = set(full_new_keys)
         current_set = set(self.keywords)
-        if full_new_set.issubset(current_set):
+        if new_keys_set.issubset(current_set):
             logging.debug('table columns before trimming: %s.',
                           ' '.join(current_set))
-            cut_keys = current_set.difference(full_new_set)
+            cut_keys = current_set.difference(new_keys_set)
             logging.debug('will try removing columns: %s.',
                           ' '.join(cut_keys))
             for key in cut_keys:
-                self._summary_info.remove_column(key)
+                self._summary.remove_column(key)
             logging.debug('after removal column names are: %s.',
                           ' '.join(self.keywords))
         else:
             logging.debug('should be building new table...')
             # Reorder the keywords to match the initial ordering.
-            new_keys.sort(key=keywords.index)
-            self._summary_info = self._fits_summary(header_keywords=new_keys)
+            new_keys_lst.sort(key=keywords.index)
+            self._summary = self._fits_summary(new_keys_lst)
+
+    @keywords.deleter
+    def keywords(self):
+        # since keywords are drawn from self._summary, setting
+        # _summary = [] deletes the keywords.
+        self._summary = []
 
     @property
     def files(self):
@@ -216,6 +314,30 @@ class ImageFileCollection(object):
         """
         return self._files
 
+    @property
+    def glob_include(self):
+        """
+        str or None, Unix-style filename pattern to select filenames to include
+        in the file collection.
+        """
+        return self._glob_include
+
+    @property
+    def glob_exclude(self):
+        """
+        str or None, Unix-style filename pattern to select filenames to exclude
+        in the file collection.
+        """
+        return self._glob_exclude
+
+    @property
+    def ext(self):
+        """
+        str or int, The extension from which the header and data will
+        be read in all files.
+        """
+        return self._ext
+
     def values(self, keyword, unique=False):
         """
         List of values for a keyword.
@@ -239,38 +361,59 @@ class ImageFileCollection(object):
                 'keyword %s is not in the current summary' % keyword)
 
         if unique:
-            return list(set(self.summary_info[keyword]))
+            return list(set(self.summary[keyword]))
         else:
-            return list(self.summary_info[keyword])
+            return list(self.summary[keyword])
 
     def files_filtered(self, **kwd):
         """Determine files whose keywords have listed values.
 
-        ``**kwd`` is list of keywords and values the files must have.
+        Parameters
+        ----------
+        include_path : bool, keyword-only
+            If the keyword ``include_path=True`` is set, the returned list
+            contains not just the filename, but the full path to each file.
+            Default is ``False``.
 
-        If the keyword ``include_path=True`` is set, the returned list
-        contains not just the filename, but the full path to each file.
+        **kwd :
+            ``**kwd`` is dict of keywords and values the files must have.
+            The value '*' represents any value.
+            A missing keyword is indicated by value ''.
 
-        The value '*' represents any value.
-        A missing keyword is indicated by value ''.
+        Returns
+        -------
+        filenames : list
+            The files that satisfy the keyword-value restrictions specified by
+            the ``**kwd``.
 
-        Example::
+        Examples
+        --------
+        Some examples for filtering::
 
             >>> keys = ['imagetyp','filter']
             >>> collection = ImageFileCollection('test/data', keywords=keys)
             >>> collection.files_filtered(imagetyp='LIGHT', filter='R')
             >>> collection.files_filtered(imagetyp='*', filter='')
 
-        NOTE: Value comparison is case *insensitive* for strings.
+        In case you want to filter with keyword names that cannot be used
+        as keyword argument name, you have to unpack them using a dictionary.
+        For example if a keyword name contains a space or a ``-``::
+
+            >>> add_filters = {'exp-time': 20, 'ESO TPL ID': 1050}
+            >>> collection.files_filtered(imagetyp='LIGHT', **add_filters)
+
+        Notes
+        -----
+        Value comparison is case *insensitive* for strings.
         """
         # force a copy by explicitly converting to a list
-        current_file_mask = list(self.summary_info['file'].mask)
+        current_file_mask = list(self.summary['file'].mask)
 
         include_path = kwd.pop('include_path', False)
 
         self._find_keywords_by_values(**kwd)
-        filtered_files = self.summary_info['file'].compressed()
-        self.summary_info['file'].mask = current_file_mask
+        filtered_files = self.summary['file'].compressed()
+        self.summary['file'].mask = current_file_mask
         if include_path:
             filtered_files = [path.join(self._location, f)
                               for f in filtered_files]
@@ -283,9 +426,9 @@ class ImageFileCollection(object):
         keywords = '*' if self._all_keywords else self.keywords
         # Re-load list of files
         self._files = self._get_files()
-        self._summary_info = self._fits_summary(header_keywords=keywords)
+        self._summary = self._fits_summary(header_keywords=keywords)
 
-    def sort(self, keys=None):
+    def sort(self, keys):
         """Sort the list of files to determine the order of iteration.
 
         Sort the table of files according to one or more keys. This does not
@@ -293,13 +436,12 @@ class ImageFileCollection(object):
 
         Parameters
         ----------
-        keys : str, list of str or None, optional
+        keys : str, list of str
             The key(s) to order the table by.
-            Default is ``None``.
         """
-        if len(self._summary_info) > 0:
-            self._summary_info.sort(keys)
-            self._files = list(self.summary_info['file'])
+        if len(self._summary) > 0:
+            self._summary.sort(keys)
+            self._files = list(self.summary['file'])
 
     def _get_files(self):
         """ Helper method which checks whether ``files`` should be set
@@ -319,6 +461,12 @@ class ImageFileCollection(object):
         else:
             files = self._fits_files_in_directory()
 
+            if self.glob_include is not None:
+                files = fnmatch.filter(files, self.glob_include)
+            if self.glob_exclude is not None:
+                files = [file for file in files
+                         if not fnmatch.fnmatch(file, self.glob_exclude)]
+
         return files
 
     def _dict_from_fits_header(self, file_name, input_summary=None,
@@ -346,10 +494,8 @@ class ImageFileCollection(object):
         -------
         file_table : `~astropy.table.Table`
         """
-        from collections import OrderedDict
 
         def _add_val_to_dict(key, value, tbl_dict, n_previous, missing_marker):
-            key = key.lower()
             try:
                 tbl_dict[key].append(value)
             except KeyError:
@@ -363,7 +509,8 @@ class ImageFileCollection(object):
             summary = input_summary
             n_previous = len(summary['file'])
 
-        h = fits.getheader(file_name)
+        h = fits.getheader(file_name, self.ext)
+
         assert 'file' not in h
 
         # Try opening header before this so that file name is only added if
@@ -379,19 +526,35 @@ class ImageFileCollection(object):
         multi_entry_keys = {'comment': [],
                             'history': []}
 
+        alreadyencountered = set()
         for k, v in six.iteritems(h):
             if k == '':
                 continue
 
-            if k.lower() in ['comment', 'history']:
-                multi_entry_keys[k.lower()].append(str(v))
+            k = k.lower()
+
+            if k in ['comment', 'history']:
+                multi_entry_keys[k].append(str(v))
                 # Accumulate these in a separate dictionary until the
                 # end to avoid adding multiple entries to summary.
                 continue
+            elif k in alreadyencountered:
+                # The "normal" multi-entries HISTORY, COMMENT and BLANK are
+                # already processed so any further duplication is probably
+                # a mistake. It would lead to problems in ImageFileCollection
+                # to add it as well, so simply ignore those.
+                warnings.warn(
+                    'Header from file "{f}" contains multiple entries for '
+                    '"{k}", the pair "{k}={v}" will be ignored.'
+                    ''.format(k=k, v=v, f=file_name),
+                    UserWarning)
+                continue
             else:
-                val = v
+                # Add the key to the already encountered keys so we don't add
+                # it more than once.
+                alreadyencountered.add(k)
 
-            _add_val_to_dict(k, val, summary, n_previous, missing_marker)
+            _add_val_to_dict(k, v, summary, n_previous, missing_marker)
 
         for k, v in six.iteritems(multi_entry_keys):
             if v:
@@ -406,26 +569,24 @@ class ImageFileCollection(object):
 
     def _set_column_name_case_to_match_keywords(self, header_keys,
                                                 summary_table):
-        key_name_dict = {k.lower(): k for k in header_keys
-                         if k != k.lower()}
+        for k in header_keys:
+            k_lower = k.lower()
+            if k_lower != k:
+                try:
+                    summary_table.rename_column(k_lower, k)
+                except KeyError:
+                    pass
 
-        for lcase, user_case in six.iteritems(key_name_dict):
-            try:
-                summary_table.rename_column(lcase, user_case)
-            except KeyError:
-                pass
-
-    def _fits_summary(self, header_keywords=None):
+    def _fits_summary(self, header_keywords):
         """
         Generate a summary table of keywords from FITS headers.
 
         Parameters
         ----------
-        header_keywords : list of str, '*' or None, optional
-            Keywords whose value should be extracted from FITS headers.
-            Default value is ``None``.
+        header_keywords : list of str or '*'
+            Keywords whose value should be extracted from FITS headers or '*'
+            to extract all.
         """
-        from astropy.table import MaskedColumn
 
         if not self.files:
             return None
@@ -440,7 +601,7 @@ class ImageFileCollection(object):
 
         file_name_column = MaskedColumn(name='file', data=self.files)
 
-        if not header_keys or (header_keys == set(['file'])):
+        if not header_keys or (header_keys == {'file'}):
             summary_table = Table(masked=True)
             summary_table.add_column(file_name_column)
             return summary_table
@@ -470,7 +631,7 @@ class ImageFileCollection(object):
         self._set_column_name_case_to_match_keywords(header_keys,
                                                      summary_table)
         missing_columns = header_keys - set(summary_table.colnames)
-        missing_columns -= set(['*'])
+        missing_columns -= {'*'}
 
         length = len(summary_table)
         for column in missing_columns:
@@ -481,7 +642,7 @@ class ImageFileCollection(object):
         if '*' not in header_keys:
             # Rearrange table columns to match order of keywords.
             # File always comes first.
-            header_keys -= set(['file'])
+            header_keys -= {'file'}
             original_order = ['file'] + sorted(header_keys,
                                                key=original_keywords.index)
             summary_table = summary_table[original_order]
@@ -512,14 +673,14 @@ class ImageFileCollection(object):
         keywords = kwd.keys()
         values = kwd.values()
 
-        if (set(keywords).issubset(set(self.keywords))):
+        if set(keywords).issubset(self.keywords):
             # we already have the information in memory
-            use_info = self.summary_info
+            use_info = self.summary
         else:
             # we need to load information about these keywords.
             use_info = self._fits_summary(header_keywords=keywords)
 
-        matches = np.array([True] * len(use_info))
+        matches = np.ones(len(use_info), dtype=bool)
         for key, value in zip(keywords, values):
             logger.debug('key %s, value %s', key, value)
             logger.debug('value in table %s', use_info[key])
@@ -532,11 +693,17 @@ class ImageFileCollection(object):
                 if isinstance(value, six.string_types):
                     # need to loop explicitly over array rather than using
                     # where to correctly do string comparison.
-                    have_this_value = np.array([False] * len(use_info))
+                    have_this_value = np.zeros(len(use_info), dtype=bool)
                     for idx, file_key_value in enumerate(use_info[key]):
                         if value_not_missing[idx]:
-                            value_matches = (file_key_value.lower() ==
-                                             value.lower())
+                            try:
+                                value_matches = (
+                                    file_key_value.lower() == value.lower())
+                            except AttributeError:
+                                # In case we're dealing with an object column
+                                # there could be values other than strings in it
+                                # so it could fail with an AttributeError.
+                                value_matches = False
                         else:
                             value_matches = False
 
@@ -557,8 +724,8 @@ class ImageFileCollection(object):
         # the numpy convention is that the mask is True for values to
         # be omitted, hence use ~matches.
         logger.debug('Matches: %s', matches)
-        self.summary_info['file'].mask = ma.nomask
-        self.summary_info['file'][~matches] = ma.masked
+        self.summary['file'].mask = ma.nomask
+        self.summary['file'].mask[~matches] = True
 
     def _fits_files_in_directory(self, extensions=None,
                                  compressed=True):
@@ -582,7 +749,6 @@ class ImageFileCollection(object):
         list
             *Names* of the files (with extension), not the full pathname.
         """
-        from .ccddata import _recognized_fits_file_extensions
 
         full_extensions = extensions or list(_recognized_fits_file_extensions)
 
@@ -652,35 +818,35 @@ class ImageFileCollection(object):
             Default is ``False``.
 
         ccd_kwargs : dict, optional
-            Dict with parameters for `~ccdproc.fits_ccddata_reader`.
+            Dict with parameters for `~astropy.nddata.fits_ccddata_reader`.
             For instance, the key ``'unit'`` can be used to specify the unit
             of the data. If ``'unit'`` is not given then ``'adu'`` is used as
             the default unit.
-            See `~ccdproc.fits_ccddata_reader` for a complete list of
+            See `~astropy.nddata.fits_ccddata_reader` for a complete list of
             parameters that can be passed through ``ccd_kwargs``.
 
-        kwd :
+        **kwd :
             Any additional keywords are used to filter the items returned; see
-            Examples for details.
+            `files_filtered` examples for details.
 
         Returns
         -------
-        {return_type}
+        `{return_type}`
             If ``return_fname`` is ``False``, yield the next {name} in the
             collection.
 
-        ({return_type}, str)
+        (`{return_type}`, str)
             If ``return_fname`` is ``True``, yield a tuple of
             ({name}, ``file name``) for the next item in the collection.
         """
         # store mask so we can reset at end--must COPY, otherwise
-        # current_mask just points to the mask of summary_info
-        if not self.summary_info:
+        # current_mask just points to the mask of summary
+        if not self.summary:
             return
 
         current_mask = {}
-        for col in self.summary_info.columns:
-            current_mask[col] = self.summary_info[col].mask
+        for col in self.summary.columns:
+            current_mask[col] = self.summary[col].mask
 
         if kwd:
             self._find_keywords_by_values(**kwd)
@@ -688,24 +854,34 @@ class ImageFileCollection(object):
         ccd_kwargs = ccd_kwargs or {}
 
         for full_path in self._paths():
-            no_scale = do_not_scale_image_data
-            hdulist = fits.open(full_path,
-                                do_not_scale_image_data=no_scale)
+            add_kwargs = {'do_not_scale_image_data': do_not_scale_image_data}
+
+            # We need to open the file here, get the appropriate values and then
+            # close it again before it "yields" otherwise it's not garantueed
+            # that the generator actually advances and closes the file again.
+            # For example if one uses "next" on the generator manually the
+            # file handle could "leak".
+            if return_type == 'header':
+                return_thing = fits.getheader(full_path, self.ext)
+            elif return_type == 'data':
+                return_thing = fits.getdata(full_path, self.ext, **add_kwargs)
+            elif return_type == 'ccd':
+                return_thing = fits_ccddata_reader(
+                    full_path, hdu=self.ext, **ccd_kwargs)
+            elif return_type == 'hdu':
+                with fits.open(full_path, **add_kwargs) as hdulist:
+                    ext_index = hdulist.index_of(self.ext)
+                    # Need to copy the HDU to prevent lazy loading problems
+                    # and "IO operations on closed file" errors
+                    return_thing = hdulist[ext_index].copy()
+            else:
+                raise ValueError('no generator for {}'.format(return_type))
 
             file_name = path.basename(full_path)
-
-            return_options = {
-                    'header': lambda: hdulist[0].header,
-                    'hdu': lambda: hdulist[0],
-                    'data': lambda: hdulist[0].data,
-                    'ccd': lambda: fits_ccddata_reader(full_path, **ccd_kwargs)
-                    }
-            try:
-                yield (return_options[return_type]()  # pragma: no branch
-                       if (not return_fname) else
-                       (return_options[return_type](), file_name))
-            except KeyError:
-                raise ValueError('no generator for {}'.format(return_type))
+            if return_fname:
+                yield return_thing, file_name
+            else:
+                yield return_thing
 
             if save_location:
                 destination_dir = save_location
@@ -721,24 +897,41 @@ class ImageFileCollection(object):
             # I really should have called the option overwrite from
             # the beginning. The hack below ensures old code works,
             # at least...
-            nuke_existing = clobber or overwrite
-            if (new_path != full_path) or nuke_existing:
-                try:
-                    hdulist.writeto(new_path, clobber=nuke_existing)
-                except IOError:
-                    logger.error('error writing file %s', new_path)
-                    raise
-            hdulist.close()
+            if clobber or overwrite:
+                if _ASTROPY_LT_1_3:
+                    nuke_existing = {'clobber': True}
+                else:
+                    nuke_existing = {'overwrite': True}
+            else:
+                nuke_existing = {}
+
+            if return_type == 'ccd':
+                pass
+            elif (new_path != full_path) or nuke_existing:
+                with fits.open(full_path, **add_kwargs) as hdulist:
+                    ext_index = hdulist.index_of(self.ext)
+                    if return_type == 'hdu':
+                        hdulist[ext_index] = return_thing
+                    elif return_type == 'data':
+                        hdulist[ext_index].data = return_thing
+                    elif return_type == 'header':
+                        hdulist[ext_index].header = return_thing
+
+                    try:
+                        hdulist.writeto(new_path, **nuke_existing)
+                    except IOError:
+                        logger.error('error writing file %s', new_path)
+                        raise
 
         # reset mask
-        for col in self.summary_info.columns:
-            self.summary_info[col].mask = current_mask[col]
+        for col in self.summary.columns:
+            self.summary[col].mask = current_mask[col]
 
     def _paths(self):
         """
         Full path to each file.
         """
-        unmasked_files = self.summary_info['file'].compressed()
+        unmasked_files = self.summary['file'].compressed()
         return [path.join(self.location, file_) for file_ in unmasked_files]
 
     def headers(self, do_not_scale_image_data=True, **kwd):
@@ -754,7 +947,8 @@ class ImageFileCollection(object):
                                do_not_scale_image_data=do_not_scale_image_data,
                                **kwd)
     hdus.__doc__ = _generator.__doc__.format(
-        name='HDU', default_scaling='False', return_type='astropy.io.fits.HDU')
+        name='HDUList', default_scaling='False',
+        return_type='astropy.io.fits.HDUList')
 
     def data(self, do_not_scale_image_data=False, **kwd):
         return self._generator('data',
@@ -764,6 +958,9 @@ class ImageFileCollection(object):
         name='image', default_scaling='False', return_type='numpy.ndarray')
 
     def ccds(self, ccd_kwargs=None, **kwd):
+        if kwd.get('clobber') or kwd.get('overwrite'):
+            raise NotImplementedError(
+                "overwrite=True (or clobber=True) is not supported for CCDs.")
         return self._generator('ccd', ccd_kwargs=ccd_kwargs, **kwd)
     ccds.__doc__ = _generator.__doc__.format(
-        name='CCDData', default_scaling='True', return_type='ccdproc.CCDData')
+        name='CCDData', default_scaling='True', return_type='astropy.nddata.CCDData')
