@@ -1,18 +1,19 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
+import gzip
 from tempfile import mkdtemp
 import os
 from shutil import rmtree
 
 import numpy as np
 
-from astropy.tests.helper import pytest
+import pytest
 from astropy import units as u
 from astropy.utils import NumpyRNGContext
 from astropy.io import fits
 
-from astropy.utils.compat import gzip
 
 from ..ccddata import CCDData
 
@@ -141,7 +142,7 @@ def triage_setup(request):
     def teardown():
         for key in n_test.keys():
             n_test[key] = 0
-        try: 
+        try:
             rmtree(test_dir)
         except OSError:
             # If we cannot clean up just keep going.
