@@ -1,22 +1,31 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-# This module implements the base CCDData class.
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import numpy as np
-from astropy.io import fits
+import textwrap
 
-from astropy.tests.helper import pytest
-from astropy.nddata import StdDevUncertainty
+import numpy as np
+import pytest
+
+from astropy.io import fits
+from astropy.nddata import StdDevUncertainty, MissingDataAssociationException
 from astropy import units as u
 from astropy.extern import six
 from astropy import log
-from astropy.wcs import WCS
+from astropy.wcs import WCS, FITSFixedWarning
+from astropy.tests.helper import catch_warnings
+from astropy.utils import minversion
+from astropy.utils.data import get_pkg_data_filename
 
 from ..ccddata import CCDData
 from .. import subtract_dark
 
 
+ASTROPY_GT_1_2 = minversion("astropy", "1.2")
+ASTROPY_GT_2_0 = minversion("astropy", "2.0")
+
+
 def test_ccddata_empty():
     with pytest.raises(TypeError):
         CCDData()  # empty initializer should fail
@@ -27,6 +36,17 @@ def test_ccddata_must_have_unit():
         CCDData(np.zeros([100, 100]))
 
 
+def test_ccddata_unit_cannot_be_set_to_none(ccd_data):
+    with pytest.raises(TypeError):
+        ccd_data.unit = None
+
+
+def test_ccddata_meta_header_conflict():
+    with pytest.raises(ValueError) as exc:
+        CCDData([1, 2, 3], unit='', meta={1: 1}, header={2: 2})
+        assert "can't have both header and meta." in str(exc)
+
+
 @pytest.mark.data_size(10)
 def test_ccddata_simple(ccd_data):
     assert ccd_data.shape == (10, 10)
@@ -48,11 +68,37 @@ def test_initialize_from_FITS(ccd_data, tmpdir):
     cd = CCDData.read(filename, unit=u.electron)
     assert cd.shape == (10, 10)
     assert cd.size == 100
-    assert np.issubdtype(cd.data.dtype, np.float)
+    assert np.issubdtype(cd.data.dtype, np.floating)
     for k, v in hdu.header.items():
         assert cd.meta[k] == v
 
 
+def test_reader_removes_wcs_related_keywords(tmpdir):
+    arr = np.arange(100).reshape(10, 10)
+    # Create a WCS programatically (straight from the astropy WCS documentation)
+    w = WCS(naxis=2)
+    w.wcs.crpix = [-234.75, 8.3393]
+    w.wcs.cdelt = np.array([-0.066667, 0.066667])
+    w.wcs.crval = [0, -90]
+    w.wcs.ctype = ["RA---AIR", "DEC--AIR"]
+    w.wcs.set_pv([(2, 1, 45.0)])
+    # Write the image including WCS to file
+    ccd = CCDData(arr, unit='adu', wcs=w)
+    filename = tmpdir.join('afile.fits').strpath
+    ccd.write(filename)
+
+    # Check that the header and the meta of the CCDData are not equal
+    ccd_fromfile = CCDData.read(filename)
+    read_header = ccd_fromfile.meta
+    ref_header = fits.getheader(filename)
+    ref_wcs = ccd_fromfile.wcs.to_header()
+    for key in read_header:
+        # Make sure no new keys were accidentally added
+        assert key in ref_header
+    for key in ref_header:
+        assert key in read_header or key in ref_wcs
+
+
 def test_initialize_from_fits_with_unit_in_header(tmpdir):
     fake_img = np.random.random(size=(100, 100))
     hdu = fits.PrimaryHDU(fake_img)
@@ -81,17 +127,22 @@ def test_initialize_from_fits_with_ADU_in_header(tmpdir):
 
 def test_initialize_from_fits_with_data_in_different_extension(tmpdir):
     fake_img = np.random.random(size=(100, 100))
-    new_hdul = fits.HDUList()
     hdu1 = fits.PrimaryHDU()
     hdu2 = fits.ImageHDU(fake_img)
     hdus = fits.HDUList([hdu1, hdu2])
     filename = tmpdir.join('afile.fits').strpath
     hdus.writeto(filename)
-    ccd = CCDData.read(filename, unit='adu')
+    with catch_warnings(FITSFixedWarning) as w:
+        ccd = CCDData.read(filename, unit='adu')
+    if not ASTROPY_GT_2_0 or minversion("astropy", "2.0.3"):
+        # Test can only succeed if astropy <= 2 (where it uses ccdprocs CCDData)
+        # or with the patched astropy.nddata.CCDData
+        # (scheduled for 2.0.3)
+        assert len(w) == 0
+        # check that the header is the combined header
+        assert hdu2.header + hdu1.header == ccd.header
     # ccd should pick up the unit adu from the fits header...did it?
     np.testing.assert_array_equal(ccd.data, fake_img)
-    # check that the header is the combined header
-    assert hdu1.header + hdu2.header == ccd.header
 
 
 def test_initialize_from_fits_with_extension(tmpdir):
@@ -109,7 +160,7 @@ def test_initialize_from_fits_with_extension(tmpdir):
     np.testing.assert_array_equal(ccd.data, fake_img2)
 
 
-def test_write_unit_to_hdu(ccd_data, tmpdir):
+def test_write_unit_to_hdu(ccd_data):
     ccd_unit = ccd_data.unit
     hdulist = ccd_data.to_hdu()
     assert 'bunit' in hdulist[0].header
@@ -117,7 +168,7 @@ def test_write_unit_to_hdu(ccd_data, tmpdir):
 
 
 def test_initialize_from_FITS_bad_keyword_raises_error(ccd_data, tmpdir):
-    # There are two fits.open keywords that are not permitted in ccdpro:
+    # There are two fits.open keywords that are not permitted in ccdproc:
     #     do_not_scale_image_data and scale_back
     filename = tmpdir.join('test.fits').strpath
     ccd_data.write(filename)
@@ -165,7 +216,7 @@ def test_fromMEF(ccd_data, tmpdir):
     np.testing.assert_array_equal(cd.data, 2 * ccd_data.data)
 
 
-def test_metafromheader(ccd_data):
+def test_metafromheader():
     hdr = fits.header.Header()
     hdr['observer'] = 'Edwin Hubble'
     hdr['exptime'] = '3600'
@@ -211,9 +262,10 @@ def test_setting_uncertainty_with_array(ccd_data):
     np.testing.assert_array_equal(ccd_data.uncertainty.array, fake_uncertainty)
 
 
-def test_setting_uncertainty_wrong_shape_raises_error(ccd_data):
+def test_setting_uncertainty_wrong_shape_raises_error():
+    ccd = CCDData(np.ones((10, 10)), unit='adu')
     with pytest.raises(ValueError):
-        ccd_data.uncertainty = np.random.random(size=2 * ccd_data.shape)
+        ccd.uncertainty = np.zeros((20, 20))
 
 
 def test_to_hdu(ccd_data):
@@ -239,7 +291,16 @@ def test_to_hdu_long_metadata_item(ccd_data):
     result = subtract_dark(ccd_data, dark, dark_exposure=30 * u.second,
                            data_exposure=15 * u.second, scale=True)
     assert 'subtract_dark' in result.header
-    hdulist = result.to_hdu()
+    with catch_warnings() as w:
+        hdulist = result.to_hdu()
+    if ASTROPY_GT_2_0:
+        # Astropys CCDData only shortens the keyword name if the value is
+        # also longer than 72 chars.
+        assert len(w) == 1
+        assert str(w[0].message) == (
+            "Keyword name 'subtract_dark' is greater than 8 characters or "
+            "contains characters not allowed by the FITS standard; a HIERARCH "
+            "card will be created.")
     header = hdulist[0].header
     assert header['subtract_dark'] == _short_names['subtract_dark']
     args_value = header[_short_names['subtract_dark']]
@@ -374,6 +435,61 @@ def test_arithmetic_overload_fails(ccd_data):
         ccd_data.subtract("five")
 
 
+def test_arithmetic_no_wcs_compare():
+    ccd = CCDData(np.ones((10, 10)), unit='')
+    assert ccd.add(ccd, compare_wcs=None).wcs is None
+    assert ccd.subtract(ccd, compare_wcs=None).wcs is None
+    assert ccd.multiply(ccd, compare_wcs=None).wcs is None
+    assert ccd.divide(ccd, compare_wcs=None).wcs is None
+
+
+@pytest.mark.skipif('not ASTROPY_GT_1_2')
+def test_arithmetic_with_wcs_compare():
+    def return_diff_smaller_3(first, second):
+        return abs(first - second) <= 3
+
+    ccd1 = CCDData(np.ones((10, 10)), unit='', wcs=2)
+    ccd2 = CCDData(np.ones((10, 10)), unit='', wcs=5)
+    assert ccd1.add(ccd2, compare_wcs=return_diff_smaller_3).wcs == 2
+    assert ccd1.subtract(ccd2, compare_wcs=return_diff_smaller_3).wcs == 2
+    assert ccd1.multiply(ccd2, compare_wcs=return_diff_smaller_3).wcs == 2
+    assert ccd1.divide(ccd2, compare_wcs=return_diff_smaller_3).wcs == 2
+
+
+@pytest.mark.skipif('not ASTROPY_GT_1_2')
+def test_arithmetic_with_wcs_compare_fail():
+    def return_diff_smaller_1(first, second):
+        return abs(first - second) <= 1
+
+    ccd1 = CCDData(np.ones((10, 10)), unit='', wcs=2)
+    ccd2 = CCDData(np.ones((10, 10)), unit='', wcs=5)
+    with pytest.raises(ValueError):
+        ccd1.add(ccd2, compare_wcs=return_diff_smaller_1).wcs
+    with pytest.raises(ValueError):
+        ccd1.subtract(ccd2, compare_wcs=return_diff_smaller_1).wcs
+    with pytest.raises(ValueError):
+        ccd1.multiply(ccd2, compare_wcs=return_diff_smaller_1).wcs
+    with pytest.raises(ValueError):
+        ccd1.divide(ccd2, compare_wcs=return_diff_smaller_1).wcs
+
+
+@pytest.mark.skipif('ASTROPY_GT_1_2')
+def test_arithmetic_with_wcs_compare_fail_astropy_version():
+    def return_diff_smaller_1(first, second):
+        return abs(first - second) <= 1
+
+    ccd1 = CCDData(np.ones((10, 10)), unit='', wcs=2)
+    ccd2 = CCDData(np.ones((10, 10)), unit='', wcs=5)
+    with pytest.raises(ImportError):
+        ccd1.add(ccd2, compare_wcs=return_diff_smaller_1).wcs
+    with pytest.raises(ImportError):
+        ccd1.subtract(ccd2, compare_wcs=return_diff_smaller_1).wcs
+    with pytest.raises(ImportError):
+        ccd1.multiply(ccd2, compare_wcs=return_diff_smaller_1).wcs
+    with pytest.raises(ImportError):
+        ccd1.divide(ccd2, compare_wcs=return_diff_smaller_1).wcs
+
+
 def test_arithmetic_overload_ccddata_operand(ccd_data):
     ccd_data.uncertainty = StdDevUncertainty(np.ones_like(ccd_data))
     operand = ccd_data.copy()
@@ -433,6 +549,44 @@ def test_arithmetic_overload_differing_units():
     assert res.unit == np.divide(a, b).unit
 
 
+@pytest.mark.skipif('not ASTROPY_GT_1_2')
+def test_arithmetic_add_with_array():
+    ccd = CCDData(np.ones((3, 3)), unit='')
+    res = ccd.add(np.arange(3))
+    np.testing.assert_array_equal(res.data, [[1, 2, 3]] * 3)
+
+    ccd = CCDData(np.ones((3, 3)), unit='adu')
+    with pytest.raises(ValueError):
+        ccd.add(np.arange(3))
+
+
+@pytest.mark.skipif('not ASTROPY_GT_1_2')
+def test_arithmetic_subtract_with_array():
+    ccd = CCDData(np.ones((3, 3)), unit='')
+    res = ccd.subtract(np.arange(3))
+    np.testing.assert_array_equal(res.data, [[1, 0, -1]] * 3)
+
+    ccd = CCDData(np.ones((3, 3)), unit='adu')
+    with pytest.raises(ValueError):
+        ccd.subtract(np.arange(3))
+
+
+@pytest.mark.skipif('not ASTROPY_GT_1_2')
+def test_arithmetic_multiply_with_array():
+    ccd = CCDData(np.ones((3, 3)) * 3, unit=u.m)
+    res = ccd.multiply(np.ones((3, 3)) * 2)
+    np.testing.assert_array_equal(res.data, [[6, 6, 6]] * 3)
+    assert res.unit == ccd.unit
+
+
+@pytest.mark.skipif('not ASTROPY_GT_1_2')
+def test_arithmetic_divide_with_array():
+    ccd = CCDData(np.ones((3, 3)), unit=u.m)
+    res = ccd.divide(np.ones((3, 3)) * 2)
+    np.testing.assert_array_equal(res.data, [[0.5, 0.5, 0.5]] * 3)
+    assert res.unit == ccd.unit
+
+
 def test_ccddata_header_does_not_corrupt_fits(ccd_data, tmpdir):
     # This test is for the problem described in astropy/ccdproc#165
     # The issue comes up when a long FITS keyword value is in a header
@@ -447,7 +601,16 @@ def test_ccddata_header_does_not_corrupt_fits(ccd_data, tmpdir):
                         data_exposure=30*u.second)
     # The write below succeeds...
     long_key = tmpdir.join('long_key.fit').strpath
-    ccd.write(long_key)
+    with catch_warnings() as w:
+        ccd.write(long_key)
+    if ASTROPY_GT_2_0:
+        # Astropys CCDData only shortens the keyword name if the value is
+        # also longer than 72 chars.
+        assert len(w) == 1
+        assert str(w[0].message) == (
+            "Keyword name 'subtract_dark' is greater than 8 characters or "
+            "contains characters not allowed by the FITS standard; a HIERARCH "
+            "card will be created.")
 
     # And this read succeeds...
     ccd_read = CCDData.read(long_key, unit="adu")
@@ -533,11 +696,6 @@ def test_wcs_attribute(ccd_data, tmpdir):
     # WCS attribute should be equal to wcs above.
     assert ccd_new.wcs.wcs == wcs.wcs
 
-    # Converting CCDData object with wcs to an hdu shouldn't
-    # create duplicate wcs-related entries in the header.
-    ccd_new_hdu = ccd_new.to_hdu()[0]
-    assert len(ccd_new_hdu.header) == original_header_length
-
     # Making a CCDData with WCS (but not WCS in the header) should lead to
     # WCS information in the header when it is converted to an HDU.
     ccd_wcs_not_in_header = CCDData(ccd_data.data, wcs=wcs, unit="adu")
@@ -587,6 +745,49 @@ def test_wcs_arithmetic_ccd(ccd_data, operation):
     assert result.wcs == ccd_data.wcs
     assert ccd_data2.wcs is None
 
+
+def test_wcs_sip_handling():
+    """
+    Check whether the ctypes RA---TAN-SIP and DEC--TAN-SIP survive
+    a roundtrip unchanged.
+    """
+    data_file = get_pkg_data_filename('data/sip-wcs.fit')
+
+    def check_wcs_ctypes_header(header):
+        expected_wcs_ctypes = {
+            'CTYPE1': 'RA---TAN-SIP',
+            'CTYPE2': 'DEC--TAN-SIP'
+        }
+        return [header[k] == v for k, v in expected_wcs_ctypes.items()]
+
+    def check_wcs_ctypes_wcs(wcs):
+        expected = ['RA---TAN-SIP', 'DEC--TAN-SIP']
+        return [act == ref for act, ref in zip(wcs.wcs.ctype, expected)]
+
+    ccd_original = CCDData.read(data_file)
+    good_ctype = check_wcs_ctypes_wcs(ccd_original.wcs)
+    assert all(good_ctype)
+
+    ccd_new = ccd_original.to_hdu()
+    good_ctype = check_wcs_ctypes_header(ccd_new[0].header)
+    assert all(good_ctype)
+
+    # Try converting to header with wcs_relax=False and
+    # the header should contain the CTYPE keywords without
+    # the -SIP
+
+    ccd_no_relax = ccd_original.to_hdu(wcs_relax=False)
+    good_ctype = check_wcs_ctypes_header(ccd_no_relax[0].header)
+    if ASTROPY_GT_1_2:
+        # This behavior was introduced in astropy 1.2.
+        assert not any(good_ctype)
+        assert ccd_no_relax[0].header['CTYPE1'] == 'RA---TAN'
+        assert ccd_no_relax[0].header['CTYPE2'] == 'DEC--TAN'
+    else:
+        # The -SIP is left in place.
+        assert all(good_ctype)
+
+
 @pytest.mark.parametrize('operation',
                          ['multiply', 'divide', 'add', 'subtract'])
 def test_mask_arithmetic_ccd(ccd_data, operation):
@@ -597,7 +798,6 @@ def test_mask_arithmetic_ccd(ccd_data, operation):
     np.testing.assert_equal(result.mask, ccd_data.mask)
 
 
-
 def test_write_read_multiextensionfits_mask_default(ccd_data, tmpdir):
     # Test that if a mask is present the mask is saved and loaded by default.
     ccd_data.mask = ccd_data.data > 10
@@ -650,7 +850,89 @@ def test_write_read_multiextensionfits_custom_ext_names(ccd_data, tmpdir):
     np.testing.assert_array_equal(ccd_data.uncertainty.array,
                                   ccd_after.uncertainty.array)
 
+
+@pytest.mark.skipif(ASTROPY_GT_2_0 and not minversion("astropy", "2.0.3"),
+                    reason="CCDData with reader is used from astropy and this "
+                           "Bug isn't fixed currently.")
+def test_read_wcs_not_creatable(tmpdir):
+    # The following Header can't be converted to a WCS object. See also
+    # astropy issue #6499.
+    hdr_txt_example_WCS = textwrap.dedent('''
+    SIMPLE  =                    T / Fits standard
+    BITPIX  =                   16 / Bits per pixel
+    NAXIS   =                    2 / Number of axes
+    NAXIS1  =                 1104 / Axis length
+    NAXIS2  =                 4241 / Axis length
+    CRVAL1  =         164.98110962 / Physical value of the reference pixel X
+    CRVAL2  =          44.34089279 / Physical value of the reference pixel Y
+    CRPIX1  =                -34.0 / Reference pixel in X (pixel)
+    CRPIX2  =               2041.0 / Reference pixel in Y (pixel)
+    CDELT1  =           0.10380000 / X Scale projected on detector (#/pix)
+    CDELT2  =           0.10380000 / Y Scale projected on detector (#/pix)
+    CTYPE1  = 'RA---TAN'           / Pixel coordinate system
+    CTYPE2  = 'WAVELENGTH'         / Pixel coordinate system
+    CUNIT1  = 'degree  '           / Units used in both CRVAL1 and CDELT1
+    CUNIT2  = 'nm      '           / Units used in both CRVAL2 and CDELT2
+    CD1_1   =           0.20760000 / Pixel Coordinate translation matrix
+    CD1_2   =           0.00000000 / Pixel Coordinate translation matrix
+    CD2_1   =           0.00000000 / Pixel Coordinate translation matrix
+    CD2_2   =           0.10380000 / Pixel Coordinate translation matrix
+    C2YPE1  = 'RA---TAN'           / Pixel coordinate system
+    C2YPE2  = 'DEC--TAN'           / Pixel coordinate system
+    C2NIT1  = 'degree  '           / Units used in both C2VAL1 and C2ELT1
+    C2NIT2  = 'degree  '           / Units used in both C2VAL2 and C2ELT2
+    RADECSYS= 'FK5     '           / The equatorial coordinate system
+    ''')
+    with catch_warnings(FITSFixedWarning):
+        hdr = fits.Header.fromstring(hdr_txt_example_WCS, sep='\n')
+    hdul = fits.HDUList([fits.PrimaryHDU(np.ones((4241, 1104)), header=hdr)])
+    filename = tmpdir.join('afile.fits').strpath
+    hdul.writeto(filename)
+    # The hdr cannot be converted to a WCS object because of an
+    # InconsistentAxisTypesError but it should still open the file
+    ccd = CCDData.read(filename, unit='adu')
+    assert ccd.wcs is None
+
+
 def test_wcs(ccd_data):
     ccd_data.wcs = 5
     assert ccd_data.wcs == 5
 
+
+def test_recognized_fits_formats():
+    from ..ccddata import _recognized_fits_file_extensions
+
+    # These are the extensions that are supposed to be supported.
+    supported_extensions = ['fit', 'fits', 'fts']
+
+    # Make sure they are actually supported.
+    assert len(set(_recognized_fits_file_extensions) -
+               set(supported_extensions)) == 0
+
+
+def test_recognized_fits_formats_for_read_write(ccd_data, tmpdir):
+    # Test that incorporates astropy/ccdproc#355, which asked that .fts
+    # be auto-identified as a FITS file extension.
+    from ..ccddata import _recognized_fits_file_extensions
+
+    for ext in _recognized_fits_file_extensions:
+        path = tmpdir.join("test.{}".format(ext))
+        ccd_data.write(path.strpath)
+        from_disk = CCDData.read(path.strpath)
+        assert (ccd_data.data == from_disk.data).all()
+
+
+def test_stddevuncertainty_compat_descriptor_no_parent():
+    with pytest.raises(MissingDataAssociationException):
+        StdDevUncertainty(np.ones((10, 10))).parent_nddata
+
+
+def test_stddevuncertainty_compat_descriptor_no_weakref():
+    # TODO: Remove this test if astropy 1.0 isn't supported anymore
+    # This test might create a Memoryleak on purpose, so the last lines after
+    # the assert are IMPORTANT cleanup.
+    ccd = CCDData(np.ones((10, 10)), unit='')
+    uncert = StdDevUncertainty(np.ones((10, 10)))
+    uncert._parent_nddata = ccd
+    assert uncert.parent_nddata is ccd
+    uncert._parent_nddata = None
