@@ -1,5 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-# This module implements the base CCDData class
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 import numpy as np
@@ -8,25 +8,29 @@ from astropy.modeling import models
 from astropy.units.quantity import Quantity
 import astropy.units as u
 from astropy.wcs import WCS
+from astropy.tests.helper import catch_warnings
+from astropy.utils.exceptions import AstropyUserWarning
 
 from astropy.nddata import StdDevUncertainty
 import astropy
 
 from numpy.testing import assert_array_equal
-from astropy.tests.helper import pytest
+import pytest
 
 from ..ccddata import CCDData
-from ..core import *
+from ..core import (
+    ccd_process, cosmicray_median, cosmicray_lacosmic, create_deviation,
+    flat_correct, gain_correct, subtract_bias, subtract_dark, subtract_overscan,
+    transform_image, trim_image, wcs_project, Keyword)
 from ..core import _blkavg
 
 try:
-    from ..core import block_reduce
+    from ..core import block_reduce, block_average, block_replicate
     HAS_BLOCK_X_FUNCS = True
-except:
+except ImportError:
     HAS_BLOCK_X_FUNCS = False
 
 
-
 # test creating deviation
 # success expected if u_image * u_gain = u_readnoise
 @pytest.mark.parametrize('u_image,u_gain,u_readnoise,expect_success', [
@@ -42,7 +46,7 @@ except:
 def test_create_deviation(ccd_data, u_image, u_gain, u_readnoise,
                           expect_success):
     ccd_data.unit = u_image
-    if u_gain:
+    if u_gain is not None:
         gain = 2.0 * u_gain
     else:
         gain = None
@@ -52,7 +56,7 @@ def test_create_deviation(ccd_data, u_image, u_gain, u_readnoise,
         assert ccd_var.uncertainty.array.shape == (10, 10)
         assert ccd_var.uncertainty.array.size == 100
         assert ccd_var.uncertainty.array.dtype == np.dtype(float)
-        if gain:
+        if gain is not None:
             expected_var = np.sqrt(2 * ccd_data.data + 5 ** 2) / 2
         else:
             expected_var = np.sqrt(ccd_data.data + 5 ** 2)
@@ -477,6 +481,41 @@ def test_flat_correct_min_value(ccd_data):
     assert flat_orig_data is not flat.data
 
 
+@pytest.mark.data_scale(10)
+def test_flat_correct_norm_value(ccd_data):
+    # Test flat correction with mean value that is different than
+    # the mean of the flat frame.
+
+    # create the flat, with some scatter
+    # Note that mean value of flat is set below and is different than
+    # the mean of the flat data.
+    flat_mean = 5.0
+    data = np.random.normal(loc=1.0, scale=0.05, size=ccd_data.shape)
+    flat = CCDData(data, meta=fits.Header(), unit=ccd_data.unit)
+    flat_data = flat_correct(ccd_data, flat, add_keyword=None,
+                             norm_value=flat_mean)
+
+    # check that the flat was normalized
+    # Should be the case that flat * flat_data = ccd_data * flat_mean
+    # if the normalization was done correctly.
+    np.testing.assert_almost_equal((flat_data.data * flat.data).mean(),
+                                   ccd_data.data.mean() * flat_mean)
+    np.testing.assert_allclose(ccd_data.data / flat_data.data,
+                               flat.data / flat_mean)
+
+
+def test_flat_correct_norm_value_bad_value(ccd_data):
+    # Test that flat_correct raises the appropriate error if
+    # it is given a bad norm_value. Bad means <=0.
+
+    # create the flat, with some scatter
+    data = np.random.normal(loc=1.0, scale=0.05, size=ccd_data.shape)
+    flat = CCDData(data, meta=fits.Header(), unit=ccd_data.unit)
+    with pytest.raises(ValueError) as e:
+        flat_correct(ccd_data, flat, add_keyword=None, norm_value=-7)
+    assert "norm_value must be" in str(e)
+
+
 # test for deviation and for flat correction
 @pytest.mark.data_scale(10)
 @pytest.mark.data_mean(300)
@@ -490,6 +529,7 @@ def test_flat_correct_deviation(ccd_data):
     flat = create_deviation(flat, readnoise=0.5 * u.electron)
     ccd_data = flat_correct(ccd_data, flat)
 
+
 # test the uncertainty on the data after flat correction
 def test_flat_correct_data_uncertainty():
     # Regression test for #345
@@ -519,7 +559,7 @@ def test_gain_correct_quantity(ccd_data):
 
 
 #test transform is ccd
-def test_transform_isccd(ccd_data):
+def test_transform_isccd():
     with pytest.raises(TypeError):
         transform_image(1, 1)
 
@@ -557,68 +597,6 @@ def test_transform_image(ccd_data, mask_data, uncertainty):
                            tran.uncertainty.array)
 
 
-#test rebinning ndarray
-def test_rebin_ndarray(ccd_data):
-    with pytest.raises(TypeError):
-        rebin(1, (5, 5))
-
-
-#test rebinning dimensions
-@pytest.mark.data_size(10)
-def test_rebin_dimensions(ccd_data):
-    with pytest.raises(ValueError):
-        rebin(ccd_data.data, (5,))
-
-
-#test rebinning dimensions
-@pytest.mark.data_size(10)
-def test_rebin_ccddata_dimensions(ccd_data):
-    with pytest.raises(ValueError):
-        rebin(ccd_data, (5,))
-
-
-#test rebinning works
-@pytest.mark.data_size(10)
-def test_rebin_larger(ccd_data):
-    a = ccd_data.data
-    b = rebin(a, (20, 20))
-
-    assert b.shape == (20, 20)
-    np.testing.assert_almost_equal(b.sum(), 4 * a.sum())
-
-
-#test rebinning is invariant
-@pytest.mark.data_size(10)
-def test_rebin_smaller(ccd_data):
-    a = ccd_data.data
-    b = rebin(a, (20, 20))
-    c = rebin(b, (10, 10))
-
-    assert c.shape == (10, 10)
-    assert (c-a).sum() == 0
-
-
-#test rebinning with ccddata object
-@pytest.mark.parametrize('mask_data, uncertainty', [
-                         (False, False),
-                         (True, True)])
-@pytest.mark.data_size(10)
-def test_rebin_ccddata(ccd_data, mask_data, uncertainty):
-    if mask_data:
-        ccd_data.mask = np.zeros_like(ccd_data)
-    if uncertainty:
-        err = np.random.normal(size=ccd_data.shape)
-        ccd_data.uncertainty = StdDevUncertainty(err)
-
-    b = rebin(ccd_data, (20, 20))
-
-    assert b.shape == (20, 20)
-    if mask_data:
-        assert b.mask.shape == (20, 20)
-    if uncertainty:
-        assert b.uncertainty.array.shape == (20, 20)
-
-
 # test block_reduce and block_replicate wrapper
 @pytest.mark.skipif(not HAS_BLOCK_X_FUNCS, reason="needs astropy >= 1.1.x")
 def test_block_reduce():
@@ -626,7 +604,10 @@ def test_block_reduce():
                   mask=np.zeros((4, 4), dtype=bool),
                   uncertainty=StdDevUncertainty(np.ones((4, 4))),
                   wcs=np.zeros((4, 4)))
-    ccd_summed = block_reduce(ccd, (2, 2))
+    with catch_warnings(AstropyUserWarning) as w:
+        ccd_summed = block_reduce(ccd, (2, 2))
+    assert len(w) == 1
+    assert 'following attributes were set' in str(w[0].message)
     assert isinstance(ccd_summed, CCDData)
     assert np.all(ccd_summed.data == 4)
     assert ccd_summed.data.shape == (2, 2)
@@ -650,7 +631,11 @@ def test_block_average():
                   uncertainty=StdDevUncertainty(np.ones((4, 4))),
                   wcs=np.zeros((4, 4)))
     ccd.data[::2, ::2] = 2
-    ccd_avgd = block_average(ccd, (2, 2))
+    with catch_warnings(AstropyUserWarning) as w:
+        ccd_avgd = block_average(ccd, (2, 2))
+    assert len(w) == 1
+    assert 'following attributes were set' in str(w[0].message)
+
     assert isinstance(ccd_avgd, CCDData)
     assert np.all(ccd_avgd.data == 1.25)
     assert ccd_avgd.data.shape == (2, 2)
@@ -673,7 +658,11 @@ def test_block_replicate():
                   mask=np.zeros((4, 4), dtype=bool),
                   uncertainty=StdDevUncertainty(np.ones((4, 4))),
                   wcs=np.zeros((4, 4)))
-    ccd_repl = block_replicate(ccd, (2, 2))
+    with catch_warnings(AstropyUserWarning) as w:
+        ccd_repl = block_replicate(ccd, (2, 2))
+    assert len(w) == 1
+    assert 'following attributes were set' in str(w[0].message)
+
     assert isinstance(ccd_repl, CCDData)
     assert np.all(ccd_repl.data == 0.25)
     assert ccd_repl.data.shape == (8, 8)
@@ -691,7 +680,7 @@ def test_block_replicate():
 
 
 #test blockaveraging ndarray
-def test__blkavg_ndarray(ccd_data):
+def test__blkavg_ndarray():
     with pytest.raises(TypeError):
         _blkavg(1, (5, 5))
 
@@ -728,50 +717,45 @@ def test_create_deviation_does_not_change_input(ccd_data):
     assert original.unit == ccd_data.unit
 
 def test_cosmicray_median_does_not_change_input(ccd_data):
-	original = ccd_data.copy()
-	error = np.zeros_like(ccd_data)
-	ccd = cosmicray_median(ccd_data,error_image = error, thresh = 5, mbox = 11, gbox = 0, rbox = 0)
-	np.testing.assert_array_equal(original.data,ccd_data.data)
-	assert original.unit == ccd_data.unit
+    original = ccd_data.copy()
+    error = np.zeros_like(ccd_data)
+    ccd = cosmicray_median(ccd_data,error_image=error, thresh=5, mbox=11, gbox=0, rbox=0)
+    np.testing.assert_array_equal(original.data,ccd_data.data)
+    assert original.unit == ccd_data.unit
 
 def test_cosmicray_lacosmic_does_not_change_input(ccd_data):
-	original = ccd_data.copy()
-	error = np.zeros_like(ccd_data)
-	ccd = cosmicray_lacosmic(ccd_data)
-	np.testing.assert_array_equal(original.data, ccd_data.data)
-	assert original.unit == ccd_data.unit
+    original = ccd_data.copy()
+    error = np.zeros_like(ccd_data)
+    ccd = cosmicray_lacosmic(ccd_data)
+    np.testing.assert_array_equal(original.data, ccd_data.data)
+    assert original.unit == ccd_data.unit
 
 def test_flat_correct_does_not_change_input(ccd_data):
-	original = ccd_data.copy()
-	flat = CCDData(np.zeros_like(ccd_data), unit = ccd_data.unit)
-	ccd = flat_correct(ccd_data,flat=flat)
-	np.testing.assert_array_equal(original.data, ccd_data.data)
-	assert original.unit == ccd_data.unit
+    original = ccd_data.copy()
+    flat = CCDData(np.zeros_like(ccd_data), unit=ccd_data.unit)
+    ccd = flat_correct(ccd_data,flat=flat)
+    np.testing.assert_array_equal(original.data, ccd_data.data)
+    assert original.unit == ccd_data.unit
 
 def test_gain_correct_does_not_change_input(ccd_data):
     original = ccd_data.copy()
-    ccd = gain_correct(ccd_data, gain = 1, gain_unit = ccd_data.unit)
+    ccd = gain_correct(ccd_data, gain=1, gain_unit=ccd_data.unit)
     np.testing.assert_array_equal(original.data, ccd_data.data)
     assert original.unit == ccd_data.unit
 
 def test_subtract_bias_does_not_change_input(ccd_data):
     original = ccd_data.copy()
-    master_frame = CCDData(np.zeros_like(ccd_data),unit = ccd_data.unit)
-    ccd = subtract_bias(ccd_data, master = master_frame)
+    master_frame = CCDData(np.zeros_like(ccd_data), unit=ccd_data.unit)
+    ccd = subtract_bias(ccd_data, master=master_frame)
     np.testing.assert_array_equal(original.data, ccd_data.data)
     assert original.unit == ccd_data.unit
 
 def test_trim_image_does_not_change_input(ccd_data):
     original = ccd_data.copy()
-    ccd = trim_image(ccd_data, fits_section = None)
+    ccd = trim_image(ccd_data, fits_section=None)
     np.testing.assert_array_equal(original.data, ccd_data.data)
     assert original.unit == ccd_data.unit
 
-def test_rebin_does_not_change_input(ccd_data):
-    original = ccd_data.copy()
-    ccd = rebin(ccd_data, (20,20))
-    np.testing.assert_array_equal(original.data, ccd_data.data)
-    assert original.unit == ccd_data.unit
 
 def test_transform_image_does_not_change_input(ccd_data):
     original = ccd_data.copy()
@@ -810,7 +794,6 @@ def test_wcs_project_onto_same_wcs(ccd_data):
     assert new_ccd.wcs.wcs.compare(target_wcs.wcs)
 
     # Make sure data matches within some reasonable tolerance.
-    print((ccd_data.data-new_ccd.data).max())
     np.testing.assert_allclose(ccd_data.data, new_ccd.data, rtol=1e-5)
 
 
@@ -984,3 +967,39 @@ def test_ccd_process():
     assert(occd.unit == u.electron)
     # Make sure the original keyword is still present. Regression test for #401
     assert occd.meta['testkw'] == 100
+
+def test_ccd_process_gain_corrected():
+    # test the through ccd_process with gain_corrected as False
+    ccd_data = CCDData(10.0 * np.ones((100, 100)), unit=u.adu)
+    ccd_data.data[:, -10:] = 2
+    ccd_data.meta['testkw'] = 100
+
+    mask = np.zeros((100, 90))
+
+    masterbias = CCDData(4.0 * np.ones((100, 90)), unit=u.adu)
+    masterbias.uncertainty = StdDevUncertainty(np.zeros((100, 90)))
+
+    dark_frame = CCDData(0.0 * np.ones((100, 90)), unit=u.adu)
+    dark_frame.uncertainty = StdDevUncertainty(np.zeros((100, 90)))
+
+    masterflat = CCDData(5.0 * np.ones((100, 90)), unit=u.adu)
+    masterflat.uncertainty = StdDevUncertainty(np.zeros((100, 90)))
+
+    occd = ccd_process(ccd_data, oscan=ccd_data[:, -10:], trim='[1:90,1:100]',
+                       error=True, master_bias=masterbias,
+                       master_flat=masterflat, dark_frame=dark_frame,
+                       bad_pixel_mask=mask, gain=0.5 * u.electron/u.adu,
+                       readnoise=5**0.5 * u.electron, oscan_median=True,
+                       dark_scale=False, dark_exposure=1.*u.s,
+                       data_exposure=1.*u.s, gain_corrected=False)
+
+    # final results should be (10 - 2) / 2.0 - 2 = 2
+    # error should be (4 + 5)**0.5 / 0.5  = 3.0
+
+    np.testing.assert_array_equal(2.0 * np.ones((100, 90)), occd.data)
+    np.testing.assert_almost_equal(3.0 * np.ones((100, 90)),
+                                  occd.uncertainty.array)
+    np.testing.assert_array_equal(mask, occd.mask)
+    assert(occd.unit == u.electron)
+    # Make sure the original keyword is still present. Regression test for #401
+    assert occd.meta['testkw'] == 100
