@@ -1,10 +1,12 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import numpy as np
 
 from astropy.extern import six
-from astropy.tests.helper import pytest
+import pytest
 
 from .. import subtract_bias, create_deviation, Keyword, CCDData
 
@@ -74,8 +76,12 @@ def test_implicit_logging(ccd_data):
     bias = CCDData(np.zeros_like(ccd_data.data), unit="adu")
     result = subtract_bias(ccd_data, bias)
     assert "subtract_bias" in result.header
-    assert result.header['subtract_bias'] == "ccd=<CCDData>, master=<CCDData>"
+    assert result.header['subtract_bias'] == (
+        'subbias', 'Shortened name for ccdproc command')
+    assert result.header['subbias'] == "ccd=<CCDData>, master=<CCDData>"
 
     result = create_deviation(ccd_data, readnoise=3 * ccd_data.unit)
+    assert result.header['create_deviation'] == (
+        'creatvar', 'Shortened name for ccdproc command')
     assert ("readnoise="+str(3 * ccd_data.unit) in
-            result.header['create_deviation'])
+            result.header['creatvar'])
