@@ -1,5 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-# This module implements the base CCDData class.
+
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
@@ -8,13 +8,12 @@ import numpy as np
 import astropy.units as u
 from astropy.stats import median_absolute_deviation as mad
 
-from astropy.tests.helper import pytest
+import pytest
 from astropy.utils.data import get_pkg_data_filename
-from astropy.utils.compat import NUMPY_LT_1_9
 from astropy.wcs import WCS
 
 from ..ccddata import CCDData
-from ..combiner import *
+from ..combiner import Combiner, combine
 
 
 #test that the Combiner raises error if empty
@@ -66,7 +65,7 @@ def test_combiner_create(ccd_data):
 #test if dtype matches the value that is passed
 def test_combiner_dtype(ccd_data):
     ccd_list = [ccd_data, ccd_data, ccd_data]
-    c = Combiner(ccd_list, dtype = np.float32)
+    c = Combiner(ccd_list, dtype=np.float32)
     assert c.data_arr.dtype == np.float32
     avg = c.average_combine()
     # dtype of average should match input dtype
@@ -74,10 +73,13 @@ def test_combiner_dtype(ccd_data):
     med = c.median_combine()
     # dtype of median should match dtype of input
     assert med.dtype == c.dtype
+    result_sum = c.sum_combine()
+    # dtype of sum should match dtype of input
+    assert result_sum.dtype == c.dtype
 
 
 #test mask is created from ccd.data
-def test_combiner_mask(ccd_data):
+def test_combiner_mask():
     data = np.zeros((10, 10))
     data[5, 5] = 1
     mask = (data == 0)
@@ -104,7 +106,7 @@ def test_weights_shape(ccd_data):
 
 
 #test the min-max rejection
-def test_combiner_minmax(ccd_data):
+def test_combiner_minmax():
     ccd_list = [CCDData(np.zeros((10, 10)), unit=u.adu),
                 CCDData(np.zeros((10, 10)) - 1000, unit=u.adu),
                 CCDData(np.zeros((10, 10)) + 1000, unit=u.adu)]
@@ -115,7 +117,7 @@ def test_combiner_minmax(ccd_data):
     assert ccd.data.mean() == 0
 
 
-def test_combiner_minmax_max(ccd_data):
+def test_combiner_minmax_max():
     ccd_list = [CCDData(np.zeros((10, 10)), unit=u.adu),
                 CCDData(np.zeros((10, 10)) - 1000, unit=u.adu),
                 CCDData(np.zeros((10, 10)) + 1000, unit=u.adu)]
@@ -125,7 +127,7 @@ def test_combiner_minmax_max(ccd_data):
     assert c.data_arr[2].mask.all()
 
 
-def test_combiner_minmax_min(ccd_data):
+def test_combiner_minmax_min():
     ccd_list = [CCDData(np.zeros((10, 10)), unit=u.adu),
                 CCDData(np.zeros((10, 10)) - 1000, unit=u.adu),
                 CCDData(np.zeros((10, 10)) + 1000, unit=u.adu)]
@@ -144,8 +146,8 @@ def test_combiner_sigmaclip_high():
                 CCDData(np.zeros((10, 10)) + 1000, unit=u.adu)]
 
     c = Combiner(ccd_list)
-    #using mad for more robust statistics vs. std
-    c.sigma_clipping(high_thresh=3, low_thresh=None, func=np.median,
+    # using mad for more robust statistics vs. std
+    c.sigma_clipping(high_thresh=3, low_thresh=None, func=np.ma.median,
                      dev_func=mad)
     assert c.data_arr[5].mask.all()
 
@@ -158,14 +160,14 @@ def test_combiner_sigmaclip_single_pix():
                 CCDData(np.zeros((10, 10)) + 10, unit=u.adu),
                 CCDData(np.zeros((10, 10)) - 10, unit=u.adu)]
     c = Combiner(ccd_list)
-    #add a single pixel in another array to check that
-    #that one gets rejected
+    # add a single pixel in another array to check that
+    # that one gets rejected
     c.data_arr[0, 5, 5] = 0
     c.data_arr[1, 5, 5] = -5
     c.data_arr[2, 5, 5] = 5
     c.data_arr[3, 5, 5] = -5
     c.data_arr[4, 5, 5] = 25
-    c.sigma_clipping(high_thresh=3, low_thresh=None, func=np.median,
+    c.sigma_clipping(high_thresh=3, low_thresh=None, func=np.ma.median,
                      dev_func=mad)
     assert c.data_arr.mask[4, 5, 5]
 
@@ -179,8 +181,8 @@ def test_combiner_sigmaclip_low():
                 CCDData(np.zeros((10, 10)) - 1000, unit=u.adu)]
 
     c = Combiner(ccd_list)
-    #using mad for more robust statistics vs. std
-    c.sigma_clipping(high_thresh=None, low_thresh=3, func=np.median,
+    # using mad for more robust statistics vs. std
+    c.sigma_clipping(high_thresh=None, low_thresh=3, func=np.ma.median,
                      dev_func=mad)
     assert c.data_arr[5].mask.all()
 
@@ -206,9 +208,19 @@ def test_combiner_average(ccd_data):
     assert ccd.unit == u.adu
     assert ccd.meta['NCOMBINE'] == len(ccd_list)
 
+#test that the sum combination works and returns a ccddata object
+def test_combiner_sum(ccd_data):
+    ccd_list = [ccd_data, ccd_data, ccd_data]
+    c = Combiner(ccd_list)
+    ccd = c.sum_combine()
+    assert isinstance(ccd, CCDData)
+    assert ccd.shape == (100, 100)
+    assert ccd.unit == u.adu
+    assert ccd.meta['NCOMBINE'] == len(ccd_list)
+
 
 #test data combined with mask is created correctly
-def test_combiner_mask_average(ccd_data):
+def test_combiner_mask_average():
     data = np.zeros((10, 10))
     data[5, 5] = 1
     mask = (data == 0)
@@ -239,7 +251,7 @@ def test_combiner_with_scaling(ccd_data):
     assert avg_ccd.shape == ccd_data.shape
     median_ccd = combiner.median_combine()
     # Does median also scale to the correct value?
-    np.testing.assert_almost_equal(np.median(median_ccd),
+    np.testing.assert_almost_equal(np.median(median_ccd.data),
                                    np.median(ccd_data.data))
 
     # Set the scaling manually...
@@ -258,7 +270,7 @@ def test_combiner_scaling_fails(ccd_data):
 
 
 #test data combined with mask is created correctly
-def test_combiner_mask_median(ccd_data):
+def test_combiner_mask_median():
     data = np.zeros((10, 10))
     data[5, 5] = 1
     mask = (data == 0)
@@ -271,6 +283,22 @@ def test_combiner_mask_median(ccd_data):
     assert ccd.mask[0, 0]
     assert not ccd.mask[5, 5]
 
+
+#test data combined with mask is created correctly
+def test_combiner_mask_sum():
+    data = np.zeros((10, 10))
+    data[5, 5] = 1
+    mask = (data == 0)
+    ccd = CCDData(data, unit=u.adu, mask=mask)
+    ccd_list = [ccd, ccd, ccd]
+    c = Combiner(ccd_list)
+    ccd = c.sum_combine()
+    assert ccd.data[0, 0] == 0
+    assert ccd.data[5, 5] == 3
+    assert ccd.mask[0, 0]
+    assert not ccd.mask[5, 5]
+
+
 #test combiner convenience function reads fits file and combine as expected
 def test_combine_average_fitsimages():
     fitsfile = get_pkg_data_filename('data/a8280271.fits')
@@ -285,6 +313,24 @@ def test_combine_average_fitsimages():
     np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data)
 
 
+def test_combine_numpyndarray():
+    """ Test of numpy ndarray implementation: #493
+
+    Test the average combine using ``Combiner`` and ``combine`` with input
+    ``img_list`` in the format of ``numpy.ndarray``.
+    """
+    fitsfile = get_pkg_data_filename('data/a8280271.fits')
+    ccd = CCDData.read(fitsfile, unit=u.adu)
+    ccd_list = [ccd]*3
+    c = Combiner(ccd_list)
+    ccd_by_combiner = c.average_combine()
+
+    fitsfilename_list = np.array([fitsfile]*3)
+    avgccd = combine(fitsfilename_list, output_file=None, method='average', unit=u.adu)
+    # averaging same fits images should give back same fits image
+    np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data)
+
+
 def test_combiner_result_dtype():
     """Regression test: #391
 
@@ -327,16 +373,14 @@ def test_combine_limitedmem_fitsimages():
     ccd_by_combiner = c.average_combine()
 
     fitsfilename_list = [fitsfile]*5
-    avgccd = combine(fitsfilename_list,output_file=None, method='average', mem_limit=1e6,
-                     unit=u.adu)
+    avgccd = combine(fitsfilename_list,output_file=None, method='average',
+                     mem_limit=1e6, unit=u.adu)
     # averaging same ccdData should give back same images
     np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data)
 
 
 #test combiner convenience function reads fits file and
 # and combine as expected when asked to run in limited memory with scaling
-@pytest.mark.xfail(NUMPY_LT_1_9,
-                   reason="numpy < 1.9 loses precision in np.ma.average")
 def test_combine_limitedmem_scale_fitsimages():
     fitsfile = get_pkg_data_filename('data/a8280271.fits')
     ccd = CCDData.read(fitsfile, unit=u.adu)
@@ -348,10 +392,10 @@ def test_combine_limitedmem_scale_fitsimages():
     ccd_by_combiner = c.average_combine()
 
     fitsfilename_list = [fitsfile]*5
-    avgccd = combine(fitsfilename_list,output_file=None, method='average',mem_limit=1e6,
-                     scale=scale_by_mean, unit=u.adu)
+    avgccd = combine(fitsfilename_list,output_file=None, method='average',
+                     mem_limit=1e6, scale=scale_by_mean, unit=u.adu)
 
-    np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data, decimal = 4)
+    np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data, decimal=4)
 
 
 #test the optional uncertainty function in average_combine
@@ -380,6 +424,19 @@ def test_median_combine_uncertainty(ccd_data):
     np.testing.assert_array_equal(ccd.data, ccd2.data)
     np.testing.assert_array_equal(ccd.uncertainty.array, ccd2.uncertainty.array)
 
+#test the optional uncertainty function in sum_combine
+def test_sum_combine_uncertainty(ccd_data):
+    ccd_list = [ccd_data, ccd_data, ccd_data]
+    c = Combiner(ccd_list)
+    ccd = c.sum_combine(uncertainty_func=np.sum)
+    uncert_ref = np.sum(c.data_arr, 0) * np.sqrt(3)
+    np.testing.assert_almost_equal(ccd.uncertainty.array, uncert_ref)
+
+    # Compare this also to the "combine" call
+    ccd2 = combine(ccd_list, method='sum', combine_uncertainty_function=np.sum)
+    np.testing.assert_array_equal(ccd.data, ccd2.data)
+    np.testing.assert_array_equal(ccd.uncertainty.array, ccd2.uncertainty.array)
+
 
 # test resulting uncertainty is corrected for the number of images
 def test_combiner_uncertainty_average():
@@ -413,6 +470,23 @@ def test_combiner_uncertainty_average_mask():
     np.testing.assert_array_almost_equal(ccd.uncertainty.array,
                                          ref_uncertainty)
 
+# test resulting uncertainty is corrected for the number of images (with mask)
+def test_combiner_uncertainty_sum_mask():
+    mask = np.zeros((10, 10), dtype=np.bool_)
+    mask[5, 5] = True
+    ccd_with_mask = CCDData(np.ones((10, 10)), unit=u.adu, mask=mask)
+    ccd_list = [ccd_with_mask,
+                CCDData(np.ones((10, 10))*2, unit=u.adu),
+                CCDData(np.ones((10, 10))*3, unit=u.adu)]
+    c = Combiner(ccd_list)
+    ccd = c.sum_combine()
+    # Just the standard deviation of ccd data.
+    ref_uncertainty = np.ones((10, 10)) * np.std([1, 2, 3])
+    ref_uncertainty *= np.sqrt(3)
+    ref_uncertainty[5, 5] = np.std([2, 3]) * np.sqrt(2)
+    np.testing.assert_array_almost_equal(ccd.uncertainty.array,
+                                         ref_uncertainty)
+
 def test_combiner_3d():
     data1 = CCDData(3 * np.ones((5,5,5)), unit=u.adu)
     data2 = CCDData(2 * np.ones((5,5,5)), unit=u.adu)
@@ -426,7 +500,7 @@ def test_combiner_3d():
 
     ccd = c.average_combine()
     assert ccd.shape == (5, 5, 5)
-    np.testing.assert_array_almost_equal(ccd.data, data1, decimal = 4)
+    np.testing.assert_array_almost_equal(ccd.data, data1, decimal=4)
 
 def test_3d_combiner_with_scaling(ccd_data):
     # The factors below are not particularly important; just avoid anything
@@ -446,7 +520,7 @@ def test_3d_combiner_with_scaling(ccd_data):
     assert avg_ccd.shape == ccd_data.shape
     median_ccd = combiner.median_combine()
     # Does median also scale to the correct value?
-    np.testing.assert_almost_equal(np.median(median_ccd),
+    np.testing.assert_almost_equal(np.median(median_ccd.data),
                                    np.median(ccd_data.data))
 
     # Set the scaling manually...
@@ -458,12 +532,12 @@ def test_3d_combiner_with_scaling(ccd_data):
 
 
 def test_clip_extrema_3d():
-    ccdlist = [CCDData(np.ones((3, 3, 3))*90., unit="adu"),\
-               CCDData(np.ones((3, 3, 3))*20., unit="adu"),\
-               CCDData(np.ones((3, 3, 3))*10., unit="adu"),\
-               CCDData(np.ones((3, 3, 3))*40., unit="adu"),\
-               CCDData(np.ones((3, 3, 3))*25., unit="adu"),\
-               CCDData(np.ones((3, 3, 3))*35., unit="adu"),\
+    ccdlist = [CCDData(np.ones((3, 3, 3))*90., unit="adu"),
+               CCDData(np.ones((3, 3, 3))*20., unit="adu"),
+               CCDData(np.ones((3, 3, 3))*10., unit="adu"),
+               CCDData(np.ones((3, 3, 3))*40., unit="adu"),
+               CCDData(np.ones((3, 3, 3))*25., unit="adu"),
+               CCDData(np.ones((3, 3, 3))*35., unit="adu"),
               ]
     c = Combiner(ccdlist)
     c.clip_extrema(nlow=1, nhigh=1)
@@ -472,7 +546,7 @@ def test_clip_extrema_3d():
     np.testing.assert_array_equal(result, expected)
 
 
-@pytest.mark.parametrize('comb_func', ['average_combine', 'median_combine'])
+@pytest.mark.parametrize('comb_func', ['average_combine', 'median_combine', 'sum_combine'])
 def test_writeable_after_combine(ccd_data, tmpdir, comb_func):
     tmp_file = tmpdir.join('tmp.fits')
     from ..combiner import Combiner
@@ -482,12 +556,12 @@ def test_writeable_after_combine(ccd_data, tmpdir, comb_func):
     ccd2.write(tmp_file.strpath)
 
 def test_clip_extrema():
-    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),\
-               CCDData(np.ones((3, 5))*20., unit="adu"),\
-               CCDData(np.ones((3, 5))*10., unit="adu"),\
-               CCDData(np.ones((3, 5))*40., unit="adu"),\
-               CCDData(np.ones((3, 5))*25., unit="adu"),\
-               CCDData(np.ones((3, 5))*35., unit="adu"),\
+    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),
+               CCDData(np.ones((3, 5))*20., unit="adu"),
+               CCDData(np.ones((3, 5))*10., unit="adu"),
+               CCDData(np.ones((3, 5))*40., unit="adu"),
+               CCDData(np.ones((3, 5))*25., unit="adu"),
+               CCDData(np.ones((3, 5))*35., unit="adu"),
               ]
     ccdlist[0].data[0,1] = 3.1
     ccdlist[1].data[1,2] = 100.1
@@ -495,36 +569,36 @@ def test_clip_extrema():
     c = Combiner(ccdlist)
     c.clip_extrema(nlow=1, nhigh=1)
     result = c.average_combine()
-    expected = [[ 30.,  22.5, 30. , 30., 30.],
-                [ 30. , 30. , 47.5, 30., 30.],
-                [ 47.5, 30. , 30. , 30., 30.]]
+    expected = [[30.0, 22.5, 30.0, 30.0, 30.0],
+                [30.0, 30.0, 47.5, 30.0, 30.0],
+                [47.5, 30.0, 30.0, 30.0, 30.0]]
     np.testing.assert_array_equal(result, expected)
 
 def test_clip_extrema_via_combine():
-    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),\
-               CCDData(np.ones((3, 5))*20., unit="adu"),\
-               CCDData(np.ones((3, 5))*10., unit="adu"),\
-               CCDData(np.ones((3, 5))*40., unit="adu"),\
-               CCDData(np.ones((3, 5))*25., unit="adu"),\
-               CCDData(np.ones((3, 5))*35., unit="adu"),\
+    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),
+               CCDData(np.ones((3, 5))*20., unit="adu"),
+               CCDData(np.ones((3, 5))*10., unit="adu"),
+               CCDData(np.ones((3, 5))*40., unit="adu"),
+               CCDData(np.ones((3, 5))*25., unit="adu"),
+               CCDData(np.ones((3, 5))*35., unit="adu"),
               ]
     ccdlist[0].data[0,1] = 3.1
     ccdlist[1].data[1,2] = 100.1
     ccdlist[1].data[2,0] = 100.1
     result = combine(ccdlist, clip_extrema=True, nlow=1, nhigh=1,)
-    expected = [[ 30.,  22.5, 30. , 30., 30.],
-                [ 30. , 30. , 47.5, 30., 30.],
-                [ 47.5, 30. , 30. , 30., 30.]]
+    expected = [[30.0, 22.5, 30.0, 30.0, 30.0],
+                [30.0, 30.0, 47.5, 30.0, 30.0],
+                [47.5, 30.0, 30.0, 30.0, 30.0]]
     np.testing.assert_array_equal(result, expected)
 
 
 def test_clip_extrema_with_other_rejection():
-    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),\
-               CCDData(np.ones((3, 5))*20., unit="adu"),\
-               CCDData(np.ones((3, 5))*10., unit="adu"),\
-               CCDData(np.ones((3, 5))*40., unit="adu"),\
-               CCDData(np.ones((3, 5))*25., unit="adu"),\
-               CCDData(np.ones((3, 5))*35., unit="adu"),\
+    ccdlist = [CCDData(np.ones((3, 5))*90., unit="adu"),
+               CCDData(np.ones((3, 5))*20., unit="adu"),
+               CCDData(np.ones((3, 5))*10., unit="adu"),
+               CCDData(np.ones((3, 5))*40., unit="adu"),
+               CCDData(np.ones((3, 5))*25., unit="adu"),
+               CCDData(np.ones((3, 5))*35., unit="adu"),
               ]
     ccdlist[0].data[0,1] = 3.1
     ccdlist[1].data[1,2] = 100.1
@@ -537,7 +611,7 @@ def test_clip_extrema_with_other_rejection():
 
     c.clip_extrema(nlow=1, nhigh=1)
     result = c.average_combine()
-    expected = [[ 80./3.,  22.5, 30. , 30., 30.],
+    expected = [[ 80./3., 22.5, 30. , 30., 30.],
                 [ 30. , 30. , 47.5, 30., 30.],
                 [ 47.5, 30. , 30. , 30., 30.]]
     np.testing.assert_array_equal(result, expected)
